@@ -47,21 +47,19 @@
 #include "File.h"
 #include "util.h"
 #include "message.h"
-#include "LogFactory.h"
-#include "Logger.h"
 #include "DlAbortEx.h"
 #include "a2io.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "DownloadFailureException.h"
+#include "error_code.h"
 
 namespace aria2 {
 
-AbstractDiskWriter::AbstractDiskWriter(const std::string& filename):
-  filename_(filename),
-  fd_(-1),
-  readOnly_(false),
-  directIOAllowed_(false),
-  logger_(LogFactory::getInstance()) {}
+AbstractDiskWriter::AbstractDiskWriter(const std::string& filename)
+  : filename_(filename),
+    fd_(-1),
+    readOnly_(false)
+{}
 
 AbstractDiskWriter::~AbstractDiskWriter()
 {
@@ -70,10 +68,14 @@ AbstractDiskWriter::~AbstractDiskWriter()
 
 void AbstractDiskWriter::openFile(uint64_t totalLength)
 {
-  if(File(filename_).exists()) {
+  try {
     openExistingFile(totalLength);
-  } else {
-    initAndOpenFile(totalLength);
+  } catch(RecoverableException& e) {
+    if(e.getErrNum() == ENOENT) {
+      initAndOpenFile(totalLength);
+    } else {
+      throw;
+    }
   }
 }
 
@@ -87,11 +89,6 @@ void AbstractDiskWriter::closeFile()
 
 void AbstractDiskWriter::openExistingFile(uint64_t totalLength)
 {
-  if(!File(filename_).exists()) {
-    throw DL_ABORT_EX
-      (StringFormat(EX_FILE_OPEN, filename_.c_str(), MSG_FILE_NOT_FOUND).str());
-  }
-
   int flags = O_BINARY;
   if(readOnly_) {
     flags |= O_RDONLY;
@@ -99,9 +96,16 @@ void AbstractDiskWriter::openExistingFile(uint64_t totalLength)
     flags |= O_RDWR;
   }
 
-  if((fd_ = open(filename_.c_str(), flags, OPEN_MODE)) < 0) {
-    throw DL_ABORT_EX
-      (StringFormat(EX_FILE_OPEN, filename_.c_str(), strerror(errno)).str());
+  while((fd_ = open(filename_.c_str(), flags, OPEN_MODE)) == -1 &&
+        errno == EINTR);
+  if(fd_ < 0) {
+    int errNum = errno;
+    throw DL_ABORT_EX3
+      (errNum,
+       fmt(EX_FILE_OPEN,
+           filename_.c_str(),
+           util::safeStrerror(errNum).c_str()),
+       error_code::FILE_OPEN_ERROR);
   }
 }
 
@@ -109,10 +113,17 @@ void AbstractDiskWriter::createFile(int addFlags)
 {
   assert(!filename_.empty());
   util::mkdirs(File(filename_).getDirname());
-  if((fd_ = open(filename_.c_str(), O_CREAT|O_RDWR|O_TRUNC|O_BINARY|addFlags,
-                OPEN_MODE)) < 0) {
-    throw DL_ABORT_EX(StringFormat(EX_FILE_OPEN,
-                                   filename_.c_str(), strerror(errno)).str());
+
+  while((fd_ = open(filename_.c_str(), O_CREAT|O_RDWR|O_TRUNC|O_BINARY|addFlags,
+                    OPEN_MODE)) == -1 && errno == EINTR);
+  if(fd_ < 0) {
+    int errNum = errno;
+    throw DL_ABORT_EX3
+      (errNum,
+       fmt(EX_FILE_OPEN,
+           filename_.c_str(),
+           util::safeStrerror(errNum).c_str()),
+       error_code::FILE_CREATE_ERROR);
   }  
 }
 
@@ -140,8 +151,11 @@ ssize_t AbstractDiskWriter::readDataInternal(unsigned char* data, size_t len)
 void AbstractDiskWriter::seek(off_t offset)
 {
   if(a2lseek(fd_, offset, SEEK_SET) == (off_t)-1) {
-    throw DL_ABORT_EX
-      (StringFormat(EX_FILE_SEEK, filename_.c_str(), strerror(errno)).str());
+    int errNum = errno;
+    throw DL_ABORT_EX2(fmt(EX_FILE_SEEK,
+                           filename_.c_str(),
+                           util::safeStrerror(errNum).c_str()),
+                       error_code::FILE_IO_ERROR);
   }
 }
 
@@ -149,14 +163,24 @@ void AbstractDiskWriter::writeData(const unsigned char* data, size_t len, off_t
 {
   seek(offset);
   if(writeDataInternal(data, len) < 0) {
+    int errNum = errno;
     // If errno is ENOSPC(not enough space in device), throw
     // DownloadFailureException and abort download instantly.
-    if(errno == ENOSPC) {
-      throw DOWNLOAD_FAILURE_EXCEPTION
-        (StringFormat(EX_FILE_WRITE, filename_.c_str(), strerror(errno)).str());
+    if(errNum == ENOSPC) {
+      throw DOWNLOAD_FAILURE_EXCEPTION3
+        (errNum,
+         fmt(EX_FILE_WRITE,
+             filename_.c_str(),
+             util::safeStrerror(errNum).c_str()),
+         error_code::NOT_ENOUGH_DISK_SPACE);
+    } else {
+      throw DL_ABORT_EX3
+        (errNum,
+         fmt(EX_FILE_WRITE,
+             filename_.c_str(),
+             util::safeStrerror(errNum).c_str()),
+         error_code::FILE_IO_ERROR);
     }
-    throw DL_ABORT_EX(StringFormat(EX_FILE_WRITE,
-                                   filename_.c_str(), strerror(errno)).str());
   }
 }
 
@@ -165,8 +189,13 @@ ssize_t AbstractDiskWriter::readData(unsigned char* data, size_t len, off_t offs
   ssize_t ret;
   seek(offset);
   if((ret = readDataInternal(data, len)) < 0) {
-    throw DL_ABORT_EX(StringFormat(EX_FILE_READ,
-                                   filename_.c_str(), strerror(errno)).str());
+    int errNum = errno;
+    throw DL_ABORT_EX3
+      (errNum,
+       fmt(EX_FILE_READ,
+           filename_.c_str(),
+           util::safeStrerror(errNum).c_str()),
+       error_code::FILE_IO_ERROR);
   }
   return ret;
 }
@@ -182,13 +211,17 @@ void AbstractDiskWriter::truncate(uint64_t length)
   HANDLE handle = LongToHandle(_get_osfhandle(fd_));
   seek(length);
   if(SetEndOfFile(handle) == 0) {
-    throw DL_ABORT_EX(StringFormat("SetEndOfFile failed. cause: %s",
-                                   GetLastError()).str());
+    throw DL_ABORT_EX2(fmt("SetEndOfFile failed. cause: %lx",
+                           GetLastError()),
+                       error_code::FILE_IO_ERROR);
   }
 #else
   if(ftruncate(fd_, length) == -1) {
-    throw DL_ABORT_EX(StringFormat("ftruncate failed. cause: %s",
-                                   strerror(errno)).str());
+    int errNum = errno;
+    throw DL_ABORT_EX3(errNum,
+                       fmt("ftruncate failed. cause: %s",
+                           util::safeStrerror(errNum).c_str()),
+                       error_code::FILE_IO_ERROR);
   }
 #endif
 }
@@ -199,19 +232,42 @@ void AbstractDiskWriter::allocate(off_t offset, uint64_t length)
   if(fd_ == -1) {
     throw DL_ABORT_EX("File not yet opened.");
   }
-# ifdef HAVE_FALLOCATE
+# ifdef __MINGW32__
+  LARGE_INTEGER fileLength;
+  fileLength.QuadPart = offset+length;
+  HANDLE handle = LongToHandle(_get_osfhandle(fd_));
+  int r;
+  r = SetFilePointerEx(handle, fileLength, 0, FILE_BEGIN);
+  if(r == 0) {
+    throw DL_ABORT_EX2(fmt("SetFilePointerEx failed. cause: %lx",
+                           GetLastError()),
+                       error_code::FILE_IO_ERROR);
+  }
+  r = SetEndOfFile(handle);
+  if(r == 0) {
+    throw DL_ABORT_EX2(fmt("SetEndOfFile failed. cause: %lx",
+                           GetLastError()),
+                       error_code::FILE_IO_ERROR);
+  }
+# elif HAVE_FALLOCATE
   // For linux, we use fallocate to detect file system supports
   // fallocate or not.
-  int r = fallocate(fd_, 0, offset, length);
+  int r;
+  while((r = fallocate(fd_, 0, offset, length)) == -1 && errno == EINTR);
+  int errNum = errno;
   if(r == -1) {
-    throw DL_ABORT_EX(StringFormat("fallocate failed. cause: %s",
-                                   strerror(errno)).str());
+    throw DL_ABORT_EX3(errNum,
+                       fmt("fallocate failed. cause: %s",
+                           util::safeStrerror(errNum).c_str()),
+                       error_code::FILE_IO_ERROR);
   }
 # elif HAVE_POSIX_FALLOCATE
   int r = posix_fallocate(fd_, offset, length);
   if(r != 0) {
-    throw DL_ABORT_EX(StringFormat("posix_fallocate failed. cause: %s",
-                                   strerror(r)).str());
+    throw DL_ABORT_EX3(r,
+                       fmt("posix_fallocate failed. cause: %s",
+                           util::safeStrerror(r).c_str()),
+                       error_code::FILE_IO_ERROR);
   }
 # else
 #  error "no *_fallocate function available."
@@ -224,26 +280,6 @@ uint64_t AbstractDiskWriter::size()
   return File(filename_).size();
 }
 
-void AbstractDiskWriter::enableDirectIO()
-{
-#ifdef ENABLE_DIRECT_IO
-  if(directIOAllowed_) {
-    int flg;
-    while((flg = fcntl(fd_, F_GETFL)) == -1 && errno == EINTR);
-    while(fcntl(fd_, F_SETFL, flg|O_DIRECT) == -1 && errno == EINTR);
-  }
-#endif // ENABLE_DIRECT_IO
-}
-
-void AbstractDiskWriter::disableDirectIO()
-{
-#ifdef ENABLE_DIRECT_IO
-  int flg;
-  while((flg = fcntl(fd_, F_GETFL)) == -1 && errno == EINTR);
-  while(fcntl(fd_, F_SETFL, flg&(~O_DIRECT)) == -1 && errno == EINTR);
-#endif // ENABLE_DIRECT_IO
-}
-
 void AbstractDiskWriter::enableReadOnly()
 {
   readOnly_ = true;
