@@ -32,18 +32,17 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_RUNTIME_H_
-#define _D_BT_RUNTIME_H_
+#ifndef D_BT_RUNTIME_H
+#define D_BT_RUNTIME_H
 
 #include "common.h"
-#include "BtConstants.h"
+#include "SharedHandle.h"
 
 namespace aria2 {
 
 class BtRuntime {
 private:
   uint64_t uploadLengthAtStartup_;
-  uint16_t port_;
   bool halt_;
   unsigned int connections_;
   bool ready_;
@@ -57,17 +56,9 @@ private:
   static const unsigned int DEFAULT_MIN_PEERS = 40;
 
 public:
-  BtRuntime():
-    uploadLengthAtStartup_(0),
-    port_(0),
-    halt_(false),
-    connections_(0),
-    ready_(false),
-    maxPeers_(DEFAULT_MAX_PEERS),
-    minPeers_(DEFAULT_MIN_PEERS)
-  {}
+  BtRuntime();
 
-  ~BtRuntime() {}
+  ~BtRuntime();
 
   uint64_t getUploadLengthAtStartup() const {
     return uploadLengthAtStartup_;
@@ -77,12 +68,6 @@ public:
     uploadLengthAtStartup_ = length;
   }
 
-  void setListenPort(uint16_t port) {
-    port_ = port;
-  }
-
-  uint16_t getListenPort() const { return port_; }
-
   bool isHalt() const { return halt_; }
 
   void setHalt(bool halt) {
@@ -114,14 +99,7 @@ public:
 
   void setReady(bool go) { ready_ = go; }
 
-  void setMaxPeers(unsigned int maxPeers)
-  {
-    maxPeers_ = maxPeers;
-    minPeers_ = static_cast<unsigned int>(maxPeers*0.8);
-    if(minPeers_ == 0 && maxPeers != 0) {
-      minPeers_ = maxPeers;
-    }
-  }
+  void setMaxPeers(unsigned int maxPeers);
 
   unsigned int getMaxPeers() const
   {
@@ -135,4 +113,4 @@ typedef SharedHandle<BtRuntime> BtRuntimeHandle;
 
 } // namespace aria2
 
-#endif // _D_BT_RUNTIME_H_
+#endif // D_BT_RUNTIME_H
