@@ -100,7 +100,8 @@ void BtSetup::setup(std::vector<Command*>& commands,
   SharedHandle<TorrentAttribute> torrentAttrs =
     bittorrent::getTorrentAttrs(requestGroup->getDownloadContext());
   bool metadataGetMode = torrentAttrs->metadata.empty();
-  BtObject btObject = e->getBtRegistry()->get(requestGroup->getGID());
+  const SharedHandle<BtRegistry>& btReg = e->getBtRegistry();
+  BtObject btObject = btReg->get(requestGroup->getGID());
   SharedHandle<PieceStorage> pieceStorage = btObject.pieceStorage_;
   SharedHandle<PeerStorage> peerStorage = btObject.peerStorage_;
   SharedHandle<BtRuntime> btRuntime = btObject.btRuntime_;
@@ -183,45 +184,40 @@ void BtSetup::setup(std::vector<Command*>& commands,
       commands.push_back(c);
     }
   }
-  if(PeerListenCommand::getNumInstance() == 0) {
+  if(btReg->getTcpPort() == 0) {
     static int families[] = { AF_INET, AF_INET6 };
     size_t familiesLength = e->getOption()->getAsBool(PREF_DISABLE_IPV6)?1:2;
     for(size_t i = 0; i < familiesLength; ++i) {
-      PeerListenCommand* listenCommand =
-        PeerListenCommand::getInstance(e, families[i]);
+      PeerListenCommand* command =
+        new PeerListenCommand(e->newCUID(), e, families[i]);
       bool ret;
       uint16_t port;
-      if(btRuntime->getListenPort()) {
-        IntSequence seq =
-          util::parseIntRange(util::uitos(btRuntime->getListenPort()));
-        ret = listenCommand->bindPort(port, seq);
+      if(btReg->getTcpPort()) {
+        IntSequence seq = util::parseIntRange(util::uitos(btReg->getTcpPort()));
+        ret = command->bindPort(port, seq);
       } else {
         IntSequence seq =
           util::parseIntRange(e->getOption()->get(PREF_LISTEN_PORT));
-        ret = listenCommand->bindPort(port, seq);
+        ret = command->bindPort(port, seq);
       }
       if(ret) {
-        btRuntime->setListenPort(port);
+        btReg->setTcpPort(port);
         // Add command to DownloadEngine directly.
-        e->addCommand(listenCommand);
+        e->addCommand(command);
       } else {
-        delete listenCommand;
+        delete command;
       }
     }
-    if(PeerListenCommand::getNumInstance() == 0) {
+    if(btReg->getTcpPort() == 0) {
       throw DL_ABORT_EX(_("Errors occurred while binding port.\n"));
     }
-  } else {
-    PeerListenCommand* listenCommand =
-      PeerListenCommand::getInstance(e, AF_INET);
-    if(!listenCommand) {
-      listenCommand = PeerListenCommand::getInstance(e, AF_INET6);
-    }
-    btRuntime->setListenPort(listenCommand->getPort());
   }
+  btAnnounce->setTcpPort(btReg->getTcpPort());
+
   if(option->getAsBool(PREF_BT_ENABLE_LPD) &&
+     btReg->getTcpPort() &&
      (metadataGetMode || !torrentAttrs->privateTorrent)) {
-    if(LpdReceiveMessageCommand::getNumInstance() == 0) {
+    if(!btReg->getLpdMessageReceiver()) {
       A2_LOG_INFO("Initializing LpdMessageReceiver.");
       SharedHandle<LpdMessageReceiver> receiver
         (new LpdMessageReceiver(LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT));
@@ -246,30 +242,30 @@ void BtSetup::setup(std::vector<Command*>& commands,
         }
       }
       if(initialized) {
+        btReg->setLpdMessageReceiver(receiver);
         A2_LOG_INFO(fmt("LpdMessageReceiver initialized. multicastAddr=%s:%u,"
                         " localAddr=%s",
                         LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT,
                         receiver->getLocalAddress().c_str()));
         LpdReceiveMessageCommand* cmd =
-          LpdReceiveMessageCommand::getInstance(e, receiver);
+          new LpdReceiveMessageCommand(e->newCUID(), receiver, e);
         e->addCommand(cmd);
       } else {
         A2_LOG_INFO("LpdMessageReceiver not initialized.");
       }
     }
-    if(LpdReceiveMessageCommand::getNumInstance()) {
+    if(btReg->getLpdMessageReceiver()) {
       const unsigned char* infoHash =
         bittorrent::getInfoHash(requestGroup->getDownloadContext());
-      SharedHandle<LpdMessageReceiver> receiver =
-        LpdReceiveMessageCommand::getInstance()->getLpdMessageReceiver();
       A2_LOG_INFO("Initializing LpdMessageDispatcher.");      
       SharedHandle<LpdMessageDispatcher> dispatcher
         (new LpdMessageDispatcher
          (std::string(&infoHash[0], &infoHash[INFO_HASH_LENGTH]),
-          btRuntime->getListenPort(),
+          btReg->getTcpPort(),
           LPD_MULTICAST_ADDR, LPD_MULTICAST_PORT));
-      if(dispatcher->init(receiver->getLocalAddress(), /*ttl*/1, /*loop*/0)) {
-        A2_LOG_INFO("LpdMessageDispatcher initialized.");      
+      if(dispatcher->init(btReg->getLpdMessageReceiver()->getLocalAddress(),
+                          /*ttl*/1, /*loop*/0)) {
+        A2_LOG_INFO("LpdMessageDispatcher initialized.");
         LpdDispatchMessageCommand* cmd =
           new LpdDispatchMessageCommand(e->newCUID(), dispatcher, e);
         cmd->setBtRuntime(btRuntime);
