@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DHT_PEER_ANNOUNCE_ENTRY_H_
-#define _D_DHT_PEER_ANNOUNCE_ENTRY_H_
+#ifndef D_DHT_PEER_ANNOUNCE_ENTRY_H
+#define D_DHT_PEER_ANNOUNCE_ENTRY_H
 
 #include "common.h"
 
@@ -93,4 +93,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_DHT_PEER_ANNOUNCE_ENTRY_H_
+#endif // D_DHT_PEER_ANNOUNCE_ENTRY_H
