@@ -66,7 +66,7 @@
 #include "PeerConnection.h"
 #include "Logger.h"
 #include "LogFactory.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "RequestGroup.h"
 #include "RequestGroupMan.h"
 #include "bittorrent_helper.h"
@@ -79,23 +79,23 @@ namespace aria2 {
 DefaultBtInteractive::DefaultBtInteractive
 (const SharedHandle<DownloadContext>& downloadContext,
  const SharedHandle<Peer>& peer)
-  :
-  downloadContext_(downloadContext),
-  peer_(peer),
-  metadataGetMode_(false),
-  logger_(LogFactory::getInstance()),
-  allowedFastSetSize_(10),
-  haveTimer_(global::wallclock),
-  keepAliveTimer_(global::wallclock),
-  floodingTimer_(global::wallclock),
-  inactiveTimer_(global::wallclock),
-  pexTimer_(global::wallclock),
-  perSecTimer_(global::wallclock),
-  keepAliveInterval_(120),
-  utPexEnabled_(false),
-  dhtEnabled_(false),
-  numReceivedMessage_(0),
-  maxOutstandingRequest_(DEFAULT_MAX_OUTSTANDING_REQUEST)
+  : downloadContext_(downloadContext),
+    peer_(peer),
+    metadataGetMode_(false),
+    localNode_(0),
+    allowedFastSetSize_(10),
+    haveTimer_(global::wallclock),
+    keepAliveTimer_(global::wallclock),
+    floodingTimer_(global::wallclock),
+    inactiveTimer_(global::wallclock),
+    pexTimer_(global::wallclock),
+    perSecTimer_(global::wallclock),
+    keepAliveInterval_(120),
+    utPexEnabled_(false),
+    dhtEnabled_(false),
+    numReceivedMessage_(0),
+    maxOutstandingRequest_(DEFAULT_MAX_OUTSTANDING_REQUEST),
+    requestGroupMan_(0)
 {}
 
 DefaultBtInteractive::~DefaultBtInteractive() {}
@@ -111,15 +111,14 @@ void DefaultBtInteractive::initiateHandshake() {
 BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
   SharedHandle<BtHandshakeMessage> message =
     btMessageReceiver_->receiveHandshake(quickReply);
-  if(message.isNull()) {
+  if(!message) {
     return SharedHandle<BtMessage>();
   }
   if(memcmp(message->getPeerId(), bittorrent::getStaticPeerId(),
             PEER_ID_LENGTH) == 0) {
     throw DL_ABORT_EX
-      (StringFormat
-       ("CUID#%s - Drop connection from the same Peer ID",
-        util::itos(cuid_).c_str()).str());
+      (fmt("CUID#%lld - Drop connection from the same Peer ID",
+           cuid_));
   }
   std::vector<SharedHandle<Peer> > activePeers;
   peerStorage_->getActivePeers(activePeers);
@@ -127,9 +126,8 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
         eoi = activePeers.end(); i != eoi; ++i) {
     if(memcmp((*i)->getPeerId(), message->getPeerId(), PEER_ID_LENGTH) == 0) {
       throw DL_ABORT_EX
-        (StringFormat
-         ("CUID#%s - Same Peer ID has been already seen.",
-          util::itos(cuid_).c_str()).str());
+        (fmt("CUID#%lld - Same Peer ID has been already seen.",
+             cuid_));
     }
   }
 
@@ -137,30 +135,22 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
     
   if(message->isFastExtensionSupported()) {
     peer_->setFastExtensionEnabled(true);
-    if(logger_->info()) {
-      logger_->info(MSG_FAST_EXTENSION_ENABLED, util::itos(cuid_).c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_FAST_EXTENSION_ENABLED, cuid_));
   }
   if(message->isExtendedMessagingEnabled()) {
     peer_->setExtendedMessagingEnabled(true);
     if(!utPexEnabled_) {
       extensionMessageRegistry_->removeExtension("ut_pex");
     }
-    if(logger_->info()) {
-      logger_->info(MSG_EXTENDED_MESSAGING_ENABLED, util::itos(cuid_).c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_EXTENDED_MESSAGING_ENABLED, cuid_));
   }
   if(message->isDHTEnabled()) {
     peer_->setDHTEnabled(true);
-    if(logger_->info()) {
-      logger_->info(MSG_DHT_ENABLED_PEER, util::itos(cuid_).c_str());
-    }
-  }
-  if(logger_->info()) {
-    logger_->info(MSG_RECEIVE_PEER_MESSAGE, util::itos(cuid_).c_str(),
-                 peer_->getIPAddress().c_str(), peer_->getPort(),
-                 message->toString().c_str());
+    A2_LOG_INFO(fmt(MSG_DHT_ENABLED_PEER, cuid_));
   }
+  A2_LOG_INFO(fmt(MSG_RECEIVE_PEER_MESSAGE, cuid_,
+                  peer_->getIPAddress().c_str(), peer_->getPort(),
+                  message->toString().c_str()));
   return message;
 }
 
@@ -200,7 +190,7 @@ void DefaultBtInteractive::addHandshakeExtendedMessageToQueue()
   static const std::string CLIENT_ARIA2("aria2/"PACKAGE_VERSION);
   HandshakeExtensionMessageHandle m(new HandshakeExtensionMessage());
   m->setClientVersion(CLIENT_ARIA2);
-  m->setTCPPort(btRuntime_->getListenPort());
+  m->setTCPPort(tcpPort_);
   m->setExtensions(extensionMessageRegistry_->getExtensions());
   SharedHandle<TorrentAttribute> attrs =
     bittorrent::getTorrentAttrs(downloadContext_);
@@ -284,21 +274,20 @@ void DefaultBtInteractive::sendKeepAlive() {
 size_t DefaultBtInteractive::receiveMessages() {
   size_t countOldOutstandingRequest = dispatcher_->countOutstandingRequest();
   size_t msgcount = 0;
-  for(int i = 0; i < 50; ++i) {
+  for(int i = 0; i < UB_MAX_OUTSTANDING_REQUEST+50; ++i) {
     if(requestGroupMan_->doesOverallDownloadSpeedExceed() ||
        downloadContext_->getOwnerRequestGroup()->doesDownloadSpeedExceed()) {
       break;
     }
     BtMessageHandle message = btMessageReceiver_->receiveMessage();
-    if(message.isNull()) {
+    if(!message) {
       break;
     }
     ++msgcount;
-    if(logger_->info()) {
-      logger_->info(MSG_RECEIVE_PEER_MESSAGE, util::itos(cuid_).c_str(),
-                   peer_->getIPAddress().c_str(), peer_->getPort(),
-                   message->toString().c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_RECEIVE_PEER_MESSAGE,
+                    cuid_,
+                    peer_->getIPAddress().c_str(), peer_->getPort(),
+                    message->toString().c_str()));
     message->doReceivedAction();
 
     switch(message->getId()) {
@@ -323,10 +312,12 @@ size_t DefaultBtInteractive::receiveMessages() {
       break;
     }
   }
-  if(countOldOutstandingRequest > 0 &&
-     dispatcher_->countOutstandingRequest() == 0){
-    maxOutstandingRequest_ = std::min((size_t)UB_MAX_OUTSTANDING_REQUEST,
-                                      maxOutstandingRequest_*2);
+  if(!pieceStorage_->isEndGame() &&
+     countOldOutstandingRequest >= maxOutstandingRequest_ &&
+     dispatcher_->countOutstandingRequest()*2 <= maxOutstandingRequest_){
+    maxOutstandingRequest_ =
+      std::min((size_t)UB_MAX_OUTSTANDING_REQUEST,
+               maxOutstandingRequest_+OUTSTANDING_REQUEST_STEP);
   }
   return msgcount;
 }
@@ -334,17 +325,13 @@ size_t DefaultBtInteractive::receiveMessages() {
 void DefaultBtInteractive::decideInterest() {
   if(pieceStorage_->hasMissingPiece(peer_)) {
     if(!peer_->amInterested()) {
-      if(logger_->debug()) {
-        logger_->debug(MSG_PEER_INTERESTED, util::itos(cuid_).c_str());
-      }
+      A2_LOG_DEBUG(fmt(MSG_PEER_INTERESTED, cuid_));
       dispatcher_->
         addMessageToQueue(messageFactory_->createInterestedMessage());
     }
   } else {
     if(peer_->amInterested()) {
-      if(logger_->debug()) {
-        logger_->debug(MSG_PEER_NOT_INTERESTED, util::itos(cuid_).c_str());
-      }
+      A2_LOG_DEBUG(fmt(MSG_PEER_NOT_INTERESTED, cuid_));
       dispatcher_->
         addMessageToQueue(messageFactory_->createNotInterestedMessage());
     }
@@ -353,46 +340,52 @@ void DefaultBtInteractive::decideInterest() {
 
 void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
   if(pieceStorage_->hasMissingPiece(peer_)) {
-
     size_t numMissingBlock = btRequestFactory_->countMissingBlock();
-
+    if(numMissingBlock >= maxMissingBlock) {
+      return;
+    }
+    size_t diffMissingBlock = maxMissingBlock-numMissingBlock;
+    std::vector<SharedHandle<Piece> > pieces;
     if(peer_->peerChoking()) {
       if(peer_->isFastExtensionEnabled()) {
-        std::vector<size_t> excludedIndexes;
-        excludedIndexes.reserve(btRequestFactory_->countTargetPiece());
-        btRequestFactory_->getTargetPieceIndexes(excludedIndexes);
-        while(numMissingBlock < maxMissingBlock) {
-          SharedHandle<Piece> piece =
-            pieceStorage_->getMissingFastPiece(peer_, excludedIndexes);
-          if(piece.isNull()) {
-            break;
-          } else {
-            btRequestFactory_->addTargetPiece(piece);
-            numMissingBlock += piece->countMissingBlock();
-            excludedIndexes.push_back(piece->getIndex());
-          }
+        if(pieceStorage_->isEndGame()) {
+          std::vector<size_t> excludedIndexes;
+          excludedIndexes.reserve(btRequestFactory_->countTargetPiece());
+          btRequestFactory_->getTargetPieceIndexes(excludedIndexes);
+          pieceStorage_->getMissingFastPiece
+            (pieces, diffMissingBlock, peer_, excludedIndexes, cuid_);
+        } else {
+          pieces.reserve(diffMissingBlock);
+          pieceStorage_->getMissingFastPiece
+            (pieces, diffMissingBlock, peer_, cuid_);
         }
       }
     } else {
-      std::vector<size_t> excludedIndexes;
-      excludedIndexes.reserve(btRequestFactory_->countTargetPiece());
-      btRequestFactory_->getTargetPieceIndexes(excludedIndexes);
-      while(numMissingBlock < maxMissingBlock) {
-        SharedHandle<Piece> piece =
-          pieceStorage_->getMissingPiece(peer_, excludedIndexes);
-        if(piece.isNull()) {
-          break;
-        } else {
-          btRequestFactory_->addTargetPiece(piece);
-          numMissingBlock += piece->countMissingBlock();
-          excludedIndexes.push_back(piece->getIndex());
-        }
+      if(pieceStorage_->isEndGame()) {
+        std::vector<size_t> excludedIndexes;
+        excludedIndexes.reserve(btRequestFactory_->countTargetPiece());
+        btRequestFactory_->getTargetPieceIndexes(excludedIndexes);
+        pieceStorage_->getMissingPiece
+          (pieces, diffMissingBlock, peer_, excludedIndexes, cuid_);
+      } else {        
+        pieces.reserve(diffMissingBlock);
+        pieceStorage_->getMissingPiece(pieces, diffMissingBlock, peer_, cuid_);
       }
     }
+    for(std::vector<SharedHandle<Piece> >::const_iterator i =
+          pieces.begin(), eoi = pieces.end(); i != eoi; ++i) {
+      btRequestFactory_->addTargetPiece(*i);
+    }
   }
 }
 
 void DefaultBtInteractive::addRequests() {
+  if(!pieceStorage_->isEndGame() && !pieceStorage_->hasMissingUnusedPiece()) {
+    pieceStorage_->enterEndGame();
+  }
+  if(pieceStorage_->isEndGame()) {
+    maxOutstandingRequest_ = 2;
+  }
   fillPiece(maxOutstandingRequest_);
   size_t reqNumToCreate =
     maxOutstandingRequest_ <= dispatcher_->countOutstandingRequest() ?
@@ -416,11 +409,9 @@ void DefaultBtInteractive::cancelAllPiece() {
       utMetadataRequestTracker_->getAllTrackedIndex();
     for(std::vector<size_t>::const_iterator i = metadataRequests.begin(),
           eoi = metadataRequests.end(); i != eoi; ++i) {
-      if(logger_->debug()) {
-        logger_->debug("Cancel metadata: piece=%lu",
-                      static_cast<unsigned long>(*i));
-      }
-      pieceStorage_->cancelPiece(pieceStorage_->getPiece(*i));
+      A2_LOG_DEBUG(fmt("Cancel metadata: piece=%lu",
+                       static_cast<unsigned long>(*i)));
+      pieceStorage_->cancelPiece(pieceStorage_->getPiece(*i), cuid_);
     }
   }
 }
@@ -451,10 +442,12 @@ void DefaultBtInteractive::checkActiveInteraction()
     const time_t interval = 30;
     if(!peer_->amInterested() && !peer_->peerInterested() &&
        inactiveTime >= interval) {
+      peer_->setDisconnectedGracefully(true);
       // TODO change the message
       throw DL_ABORT_EX
-        (StringFormat("Disconnect peer because we are not interested each other"
-                      " after %u second(s).", interval).str());
+        (fmt("Disconnect peer because we are not interested each other"
+             " after %ld second(s).",
+             static_cast<long int>(interval)));
     }
   }
   // Since the peers which are *just* connected and do nothing to improve
@@ -463,10 +456,16 @@ void DefaultBtInteractive::checkActiveInteraction()
   {
     const time_t interval = 60;
     if(inactiveTime >= interval) {
+      peer_->setDisconnectedGracefully(true);
       throw DL_ABORT_EX
-        (StringFormat(EX_DROP_INACTIVE_CONNECTION, interval).str());
+        (fmt(EX_DROP_INACTIVE_CONNECTION,
+             static_cast<long int>(interval)));
     }
   }
+  // If both of us are seeders, drop connection.
+  if(peer_->isSeeder() && pieceStorage_->downloadFinished()) {
+    throw DL_ABORT_EX(MSG_GOOD_BYE_SEEDER);
+  }
 }
 
 void DefaultBtInteractive::addPeerExchangeMessage()
@@ -475,26 +474,27 @@ void DefaultBtInteractive::addPeerExchangeMessage()
      difference(global::wallclock) >= UTPexExtensionMessage::DEFAULT_INTERVAL) {
     UTPexExtensionMessageHandle m
       (new UTPexExtensionMessage(peer_->getExtensionMessageID("ut_pex")));
-    const std::deque<SharedHandle<Peer> >& peers = peerStorage_->getPeers();
-    {
-      for(std::deque<SharedHandle<Peer> >::const_iterator i =
-            peers.begin(), eoi = peers.end();
-          i != eoi && !m->freshPeersAreFull(); ++i) {
-        if(peer_->getIPAddress() != (*i)->getIPAddress()) {
-          m->addFreshPeer(*i);
-        }
+
+    std::vector<SharedHandle<Peer> > activePeers;
+    peerStorage_->getActivePeers(activePeers);
+    for(std::vector<SharedHandle<Peer> >::const_iterator i =
+          activePeers.begin(), eoi = activePeers.end();
+        i != eoi && !m->freshPeersAreFull(); ++i) {
+      if(peer_->getIPAddress() != (*i)->getIPAddress()) {
+        m->addFreshPeer(*i);
       }
     }
-    {
-      for(std::deque<SharedHandle<Peer> >::const_reverse_iterator i =
-            peers.rbegin(), eoi = peers.rend();
-          i != eoi && !m->droppedPeersAreFull();
-          ++i) {
-        if(peer_->getIPAddress() != (*i)->getIPAddress()) {
-          m->addDroppedPeer(*i);
-        }
+    const std::deque<SharedHandle<Peer> >& droppedPeers =
+      peerStorage_->getDroppedPeers();
+    for(std::deque<SharedHandle<Peer> >::const_iterator i =
+          droppedPeers.begin(), eoi = droppedPeers.end();
+        i != eoi && !m->droppedPeersAreFull();
+        ++i) {
+      if(peer_->getIPAddress() != (*i)->getIPAddress()) {
+        m->addDroppedPeer(*i);
       }
     }
+
     BtMessageHandle msg = messageFactory_->createBtExtendedMessage(m);
     dispatcher_->addMessageToQueue(msg);
     pexTimer_ = global::wallclock;
@@ -525,7 +525,7 @@ void DefaultBtInteractive::doInteractionProcessing() {
           utMetadataRequestTracker_->removeTimeoutEntry();
         for(std::vector<size_t>::const_iterator i = indexes.begin(),
               eoi = indexes.end(); i != eoi; ++i) {
-          pieceStorage_->cancelPiece(pieceStorage_->getPiece(*i));
+          pieceStorage_->cancelPiece(pieceStorage_->getPiece(*i), cuid_);
         }
       }
       if(pieceStorage_->downloadFinished()) {
@@ -557,7 +557,7 @@ void DefaultBtInteractive::doInteractionProcessing() {
   sendPendingMessage();
 }
 
-void DefaultBtInteractive::setLocalNode(const WeakHandle<DHTNode>& node)
+void DefaultBtInteractive::setLocalNode(DHTNode* node)
 {
   localNode_ = node;
 }
@@ -645,8 +645,7 @@ void DefaultBtInteractive::setBtMessageFactory
   messageFactory_ = factory;
 }
 
-void DefaultBtInteractive::setRequestGroupMan
-(const WeakHandle<RequestGroupMan>& rgman)
+void DefaultBtInteractive::setRequestGroupMan(RequestGroupMan* rgman)
 {
   requestGroupMan_ = rgman;
 }
