@@ -32,18 +32,18 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DEFAULT_PEER_STORAGE_H_
-#define _D_DEFAULT_PEER_STORAGE_H_
+#ifndef D_DEFAULT_PEER_STORAGE_H
+#define D_DEFAULT_PEER_STORAGE_H
 
 #include "PeerStorage.h"
 
+#include <string>
 #include <map>
 
 #include "TimerA2.h"
 
 namespace aria2 {
 
-class Logger;
 class BtRuntime;
 class BtSeederStateChoke;
 class BtLeecherStateChoke;
@@ -53,8 +53,9 @@ class DefaultPeerStorage : public PeerStorage {
 private:
   SharedHandle<BtRuntime> btRuntime_;
   SharedHandle<PieceStorage> pieceStorage_;
+  size_t maxPeerListSize_;
   std::deque<SharedHandle<Peer> > peers_;
-  Logger* logger_;
+  std::deque<SharedHandle<Peer> > droppedPeers_;
   uint64_t removedPeerSessionDownloadLength_;
   uint64_t removedPeerSessionUploadLength_;
 
@@ -68,6 +69,8 @@ private:
   TransferStat cachedTransferStat_;
 
   bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
+
+  void addDroppedPeer(const SharedHandle<Peer>& peer);
 public:
   DefaultPeerStorage();
 
@@ -75,7 +78,7 @@ public:
 
   virtual bool addPeer(const SharedHandle<Peer>& peer);
 
-  size_t countPeer() const;
+  virtual size_t countPeer() const;
 
   virtual SharedHandle<Peer> getUnusedPeer();
 
@@ -85,6 +88,8 @@ public:
 
   virtual const std::deque<SharedHandle<Peer> >& getPeers();
 
+  virtual const std::deque<SharedHandle<Peer> >& getDroppedPeers();
+
   virtual bool isPeerAvailable();
 
   virtual void getActivePeers(std::vector<SharedHandle<Peer> >& peers);
@@ -110,8 +115,13 @@ public:
   void setPieceStorage(const SharedHandle<PieceStorage>& pieceStorage);
 
   void setBtRuntime(const SharedHandle<BtRuntime>& btRuntime);
+
+  void setMaxPeerListSize(size_t maxPeerListSize)
+  {
+    maxPeerListSize_ = maxPeerListSize;
+  }
 };
 
 } // namespace aria2
 
-#endif // _D_DEFAULT_PEER_STORAGE_H_
+#endif // D_DEFAULT_PEER_STORAGE_H
