@@ -54,8 +54,10 @@
 #include "util.h"
 #include "a2functional.h"
 #include "Option.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "RarestPieceSelector.h"
+#include "DefaultStreamPieceSelector.h"
+#include "InorderStreamPieceSelector.h"
 #include "array_fun.h"
 #include "PieceStatMan.h"
 #include "wallclock.h"
@@ -67,42 +69,51 @@
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
-(const SharedHandle<DownloadContext>& downloadContext, const Option* option):
-  downloadContext_(downloadContext),
-  bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
-                               downloadContext->getTotalLength())),
-  diskWriterFactory_(new DefaultDiskWriterFactory()),
-  endGame_(false),
-  endGamePieceNum_(END_GAME_PIECE_NUM),
-  logger_(LogFactory::getInstance()),
-  option_(option),
-  pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
-  pieceSelector_(new RarestPieceSelector(pieceStatMan_))
-{}
+(const SharedHandle<DownloadContext>& downloadContext, const Option* option)
+ : downloadContext_(downloadContext),
+   bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
+                                downloadContext->getTotalLength())),
+   diskWriterFactory_(new DefaultDiskWriterFactory()),
+   endGame_(false),
+   endGamePieceNum_(END_GAME_PIECE_NUM),
+   option_(option),
+   pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
+   pieceSelector_(new RarestPieceSelector(pieceStatMan_))
+{
+  const std::string& pieceSelectorOpt =
+    option_->get(PREF_STREAM_PIECE_SELECTOR);
+  if(pieceSelectorOpt.empty() || pieceSelectorOpt == A2_V_DEFAULT) {
+    streamPieceSelector_ = SharedHandle<StreamPieceSelector>
+      (new DefaultStreamPieceSelector(bitfieldMan_));
+  } else if(pieceSelectorOpt == V_INORDER) {
+    streamPieceSelector_ = SharedHandle<StreamPieceSelector>
+      (new InorderStreamPieceSelector(bitfieldMan_));
+  }
+}
 
-DefaultPieceStorage::~DefaultPieceStorage() {
+DefaultPieceStorage::~DefaultPieceStorage()
+{
   delete bitfieldMan_;
 }
 
-SharedHandle<Piece> DefaultPieceStorage::checkOutPiece(size_t index)
+SharedHandle<Piece> DefaultPieceStorage::checkOutPiece
+(size_t index, cuid_t cuid)
 {
   bitfieldMan_->setUseBit(index);
 
   SharedHandle<Piece> piece = findUsedPiece(index);
-  if(piece.isNull()) {
+  if(!piece) {
     piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
-
 #ifdef ENABLE_MESSAGE_DIGEST
 
-    piece->setHashAlgo(downloadContext_->getPieceHashAlgo());
+    piece->setHashType(downloadContext_->getPieceHashType());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
     addUsedPiece(piece);
-    return piece;
-  } else {
-    return piece;
   }
+  piece->addUser(cuid);
+  return piece;
 }
 
 /**
@@ -114,7 +125,7 @@ SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
   SharedHandle<Piece> piece;
   if(0 <= index && index <= bitfieldMan_->getMaxIndex()) {
     piece = findUsedPiece(index);
-    if(piece.isNull()) {
+    if(!piece) {
       piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
       if(hasPiece(index)) {
         piece->setAllBlock();
@@ -127,12 +138,11 @@ SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
 void DefaultPieceStorage::addUsedPiece(const SharedHandle<Piece>& piece)
 {
   std::deque<SharedHandle<Piece> >::iterator i =
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece,
+                     DerefLess<SharedHandle<Piece> >());
   usedPieces_.insert(i, piece);
-  if(logger_->debug()) {
-    logger_->debug("usedPieces_.size()=%lu",
-                   static_cast<unsigned long>(usedPieces_.size()));
-  }
+  A2_LOG_DEBUG(fmt("usedPieces_.size()=%lu",
+                   static_cast<unsigned long>(usedPieces_.size())));
 }
 
 SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
@@ -141,11 +151,12 @@ SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
   p->setIndex(index);
 
   std::deque<SharedHandle<Piece> >::const_iterator i =
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), p);
-  if(i != usedPieces_.end() && (*i) == p) {
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), p,
+                     DerefLess<SharedHandle<Piece> >());
+  if(i != usedPieces_.end() && *(*i) == *p) {
     return *i;
   } else {
-    p.reset(0);
+    p.reset();
     return p;
   }
 }
@@ -162,7 +173,8 @@ void DefaultPieceStorage::getMissingPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
  const unsigned char* bitfield,
- size_t length)
+ size_t length,
+ cuid_t cuid)
 {
   const size_t mislen = bitfieldMan_->getBitfieldLength();
   array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
@@ -183,8 +195,14 @@ void DefaultPieceStorage::getMissingPiece
     std::random_shuffle(indexes.begin(), indexes.end());
     for(std::vector<size_t>::const_iterator i = indexes.begin(),
           eoi = indexes.end(); i != eoi && misBlock < minMissingBlocks; ++i) {
-      pieces.push_back(checkOutPiece(*i));
-      misBlock += pieces.back()->countMissingBlock();
+      SharedHandle<Piece> piece = checkOutPiece(*i, cuid);
+      if(piece->getUsedBySegment()) {
+        // We don't share piece downloaded via HTTP/FTP
+        piece->removeUser(cuid);
+      } else {
+        pieces.push_back(piece);
+        misBlock += piece->countMissingBlock();
+      }
     }
   } else {
     bool r = bitfieldMan_->getAllMissingUnusedIndexes
@@ -195,7 +213,7 @@ void DefaultPieceStorage::getMissingPiece
     while(misBlock < minMissingBlocks) {
       size_t index;
       if(pieceSelector_->select(index, misbitfield, blocks)) {
-        pieces.push_back(checkOutPiece(index));
+        pieces.push_back(checkOutPiece(index, cuid));
         bitfield::flipBit(misbitfield, blocks, index);
         misBlock += pieces.back()->countMissingBlock();
       } else {
@@ -229,10 +247,12 @@ void DefaultPieceStorage::createFastIndexBitfield
 void DefaultPieceStorage::getMissingPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const SharedHandle<Peer>& peer,
+ cuid_t cuid)
 {
   getMissingPiece(pieces, minMissingBlocks,
-                  peer->getBitfield(), peer->getBitfieldLength());
+                  peer->getBitfield(), peer->getBitfieldLength(),
+                  cuid);
 }
 
 
@@ -240,20 +260,23 @@ void DefaultPieceStorage::getMissingPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
  const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
                            bitfieldMan_->getTotalLength());
   tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
   unsetExcludedIndexes(tempBitfield, excludedIndexes);
   getMissingPiece(pieces, minMissingBlocks,
-                  tempBitfield.getBitfield(), tempBitfield.getBitfieldLength());
+                  tempBitfield.getBitfield(), tempBitfield.getBitfieldLength(),
+                  cuid);
 }
 
 void DefaultPieceStorage::getMissingFastPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const SharedHandle<Peer>& peer,
+ cuid_t cuid)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
@@ -261,7 +284,8 @@ void DefaultPieceStorage::getMissingFastPiece
     createFastIndexBitfield(tempBitfield, peer);
     getMissingPiece(pieces, minMissingBlocks,
                     tempBitfield.getBitfield(),
-                    tempBitfield.getBitfieldLength());
+                    tempBitfield.getBitfieldLength(),
+                    cuid);
   }
 }
 
@@ -269,7 +293,8 @@ void DefaultPieceStorage::getMissingFastPiece
 (std::vector<SharedHandle<Piece> >& pieces,
  size_t minMissingBlocks,
  const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
@@ -278,15 +303,18 @@ void DefaultPieceStorage::getMissingFastPiece
     unsetExcludedIndexes(tempBitfield, excludedIndexes);
     getMissingPiece(pieces, minMissingBlocks,
                     tempBitfield.getBitfield(),
-                    tempBitfield.getBitfieldLength());
+                    tempBitfield.getBitfieldLength(),
+                    cuid);
   }
 }
 
 SharedHandle<Piece>
-DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+DefaultPieceStorage::getMissingPiece
+(const SharedHandle<Peer>& peer,
+ cuid_t cuid)
 {
   std::vector<SharedHandle<Piece> > pieces;
-  getMissingPiece(pieces, 1, peer);
+  getMissingPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
     return SharedHandle<Piece>();
   } else {
@@ -295,10 +323,12 @@ DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+(const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   std::vector<SharedHandle<Piece> > pieces;
-  getMissingPiece(pieces, 1, peer, excludedIndexes);
+  getMissingPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
     return SharedHandle<Piece>();
   } else {
@@ -307,10 +337,11 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer)
+(const SharedHandle<Peer>& peer,
+ cuid_t cuid)
 {
   std::vector<SharedHandle<Piece> > pieces;
-  getMissingFastPiece(pieces, 1, peer);
+  getMissingFastPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
     return SharedHandle<Piece>();
   } else {
@@ -319,10 +350,12 @@ SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
 }
 
 SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+(const SharedHandle<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   std::vector<SharedHandle<Piece> > pieces;
-  getMissingFastPiece(pieces, 1, peer, excludedIndexes);
+  getMissingFastPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
     return SharedHandle<Piece>();
   } else {
@@ -338,35 +371,41 @@ bool DefaultPieceStorage::hasMissingUnusedPiece()
   return bitfieldMan_->getFirstMissingUnusedIndex(index);
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getSparseMissingUnusedPiece
-(size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length)
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+(size_t minSplitSize,
+ const unsigned char* ignoreBitfield,
+ size_t length,
+ cuid_t cuid)
 {
   size_t index;
-  if(bitfieldMan_->getSparseMissingUnusedIndex
+  if(streamPieceSelector_->select
      (index, minSplitSize, ignoreBitfield, length)) {
-    return checkOutPiece(index);
+    return checkOutPiece(index, cuid);
   } else {
     return SharedHandle<Piece>();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece(size_t index)
+SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
+(size_t index,
+ cuid_t cuid)
 {
   if(hasPiece(index) || isPieceUsed(index)) {
     return SharedHandle<Piece>();
   } else {
-    return checkOutPiece(index);
+    return checkOutPiece(index, cuid);
   }
 }
 
 void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
   std::deque<SharedHandle<Piece> >::iterator i = 
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
-  if(i != usedPieces_.end() && (*i) == piece) {
+    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece,
+                     DerefLess<SharedHandle<Piece> >());
+  if(i != usedPieces_.end() && *(*i) == *piece) {
     usedPieces_.erase(i);
   }
 }
@@ -409,7 +448,7 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 
 void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
   deleteUsedPiece(piece);
@@ -425,12 +464,12 @@ void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
   if(downloadFinished()) {
     downloadContext_->resetDownloadStopTime();
     if(isSelectiveDownloadingMode()) {
-      logger_->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
+      A2_LOG_NOTICE(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
       // message after user-specified files were downloaded.
       //finishSelectiveDownloadingMode();
     } else {
-      logger_->info(MSG_DOWNLOAD_COMPLETED);
+      A2_LOG_INFO(MSG_DOWNLOAD_COMPLETED);
     }
 #ifdef ENABLE_BITTORRENT
     if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
@@ -451,12 +490,16 @@ bool DefaultPieceStorage::isSelectiveDownloadingMode()
 }
 
 // not unittested
-void DefaultPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::cancelPiece
+(const SharedHandle<Piece>& piece, cuid_t cuid)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
-  bitfieldMan_->unsetUseBit(piece->getIndex());
+  piece->removeUser(cuid);
+  if(!piece->getUsed()) {
+    bitfieldMan_->unsetUseBit(piece->getIndex());
+  }
   if(!isEndGame()) {
     if(piece->getCompletedLength() == 0) {
       deleteUsedPiece(piece);
@@ -558,9 +601,7 @@ bool DefaultPieceStorage::allDownloadFinished()
 void DefaultPieceStorage::initStorage()
 {
   if(downloadContext_->getFileEntries().size() == 1) {
-    if(logger_->debug()) {
-      logger_->debug("Instantiating DirectDiskAdaptor");
-    }
+    A2_LOG_DEBUG("Instantiating DirectDiskAdaptor");
     DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
     directDiskAdaptor->setTotalLength(downloadContext_->getTotalLength());
     directDiskAdaptor->setFileEntries
@@ -569,22 +610,13 @@ void DefaultPieceStorage::initStorage()
 
     DiskWriterHandle writer =
       diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath());
-    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
-      writer->allowDirectIO();
-    }
-
     directDiskAdaptor->setDiskWriter(writer);
     diskAdaptor_ = directDiskAdaptor;
   } else {
-    if(logger_->debug()) {
-      logger_->debug("Instantiating MultiDiskAdaptor");
-    }
+    A2_LOG_DEBUG("Instantiating MultiDiskAdaptor");
     MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
     multiDiskAdaptor->setFileEntries(downloadContext_->getFileEntries().begin(),
                                      downloadContext_->getFileEntries().end());
-    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
-      multiDiskAdaptor->allowDirectIO();
-    }
     multiDiskAdaptor->setPieceLength(downloadContext_->getPieceLength());
     multiDiskAdaptor->setMaxOpenFiles
       (option_->getAsInt(PREF_BT_MAX_OPEN_FILES));
@@ -668,10 +700,8 @@ void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
   std::deque<HaveEntry>::iterator itr =
     std::find_if(haves_.begin(), haves_.end(), FindElapsedHave(elapsed));
   if(itr != haves_.end()) {
-    if(logger_->debug()) {
-      logger_->debug(MSG_REMOVED_HAVE_ENTRY,
-                     static_cast<unsigned long>(haves_.end()-itr));
-    }
+    A2_LOG_DEBUG(fmt(MSG_REMOVED_HAVE_ENTRY,
+                     static_cast<unsigned long>(haves_.end()-itr)));
     haves_.erase(itr, haves_.end());
   }
 }
@@ -705,7 +735,7 @@ void DefaultPieceStorage::markPiecesDone(uint64_t length)
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-      p->setHashAlgo(downloadContext_->getPieceHashAlgo());
+      p->setHashType(downloadContext_->getPieceHashType());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -723,7 +753,8 @@ void DefaultPieceStorage::addInFlightPiece
 (const std::vector<SharedHandle<Piece> >& pieces)
 {
   usedPieces_.insert(usedPieces_.end(), pieces.begin(), pieces.end());
-  std::sort(usedPieces_.begin(), usedPieces_.end());
+  std::sort(usedPieces_.begin(), usedPieces_.end(),
+            DerefLess<SharedHandle<Piece> >());
 }
 
 size_t DefaultPieceStorage::countInFlightPiece()
