@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DEFAULT_PIECE_STORAGE_H_
-#define _D_DEFAULT_PIECE_STORAGE_H_
+#ifndef D_DEFAULT_PIECE_STORAGE_H
+#define D_DEFAULT_PIECE_STORAGE_H
 
 #include "PieceStorage.h"
 
@@ -43,12 +43,12 @@ namespace aria2 {
 
 class DownloadContext;
 class BitfieldMan;
-class Logger;
 class Option;
 class DiskWriterFactory;
 class FileEntry;
 class PieceStatMan;
 class PieceSelector;
+class StreamPieceSelector;
 
 #define END_GAME_PIECE_NUM 20
 
@@ -78,29 +78,28 @@ private:
   SharedHandle<DiskWriterFactory> diskWriterFactory_;
   std::deque<SharedHandle<Piece> > usedPieces_;
 
+  bool endGame_;
   size_t endGamePieceNum_;
-  Logger* logger_;
   const Option* option_;
   std::deque<HaveEntry> haves_;
 
   SharedHandle<PieceStatMan> pieceStatMan_;
 
   SharedHandle<PieceSelector> pieceSelector_;
-
-  bool getMissingPieceIndex(size_t& index,
-                            const unsigned char* bitfield, size_t length);
-
-  SharedHandle<Piece> getMissingPiece(const unsigned char* bitfield,
-                                      size_t length);
-
-  SharedHandle<Piece> getMissingPiece(const BitfieldMan& bitfield);
-
+  SharedHandle<StreamPieceSelector> streamPieceSelector_;
 #ifdef ENABLE_BITTORRENT
+  void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const unsigned char* bitfield,
+   size_t length,
+   cuid_t cuid);
+
   void createFastIndexBitfield(BitfieldMan& bitfield,
                                const SharedHandle<Peer>& peer);
 #endif // ENABLE_BITTORRENT
 
-  SharedHandle<Piece> checkOutPiece(size_t index);
+  SharedHandle<Piece> checkOutPiece(size_t index, cuid_t cuid);
   //   size_t deleteUsedPiecesByFillRate(int fillRate, size_t toDelete);
   //   void reduceUsedPieces(size_t upperBound);
   void deleteUsedPiece(const SharedHandle<Piece>& piece);
@@ -122,31 +121,67 @@ public:
 
   virtual bool hasMissingPiece(const SharedHandle<Peer>& peer);
 
-  virtual SharedHandle<Piece> getMissingPiece(const SharedHandle<Peer>& peer);
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid);
 
-  virtual SharedHandle<Piece> getMissingFastPiece
-  (const SharedHandle<Peer>& peer);
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid);
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
 
   virtual SharedHandle<Piece> getMissingPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes);
+  (const SharedHandle<Peer>& peer,
+   cuid_t cuid);
 
-  virtual SharedHandle<Piece> getMissingFastPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes);
+  virtual  SharedHandle<Piece> getMissingPiece
+  (const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
+
+  SharedHandle<Piece> getMissingFastPiece
+  (const SharedHandle<Peer>& peer,
+   cuid_t cuid);
+
+  SharedHandle<Piece> getMissingFastPiece
+  (const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
 
 #endif // ENABLE_BITTORRENT
 
   virtual bool hasMissingUnusedPiece();
 
-  virtual SharedHandle<Piece> getSparseMissingUnusedPiece
-  (size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length);
+  virtual SharedHandle<Piece> getMissingPiece
+  (size_t minSplitSize,
+   const unsigned char* ignoreBitfield,
+   size_t length,
+   cuid_t cuid);
 
-  virtual SharedHandle<Piece> getMissingPiece(size_t index);
+  virtual SharedHandle<Piece> getMissingPiece(size_t index, cuid_t cuid);
 
   virtual SharedHandle<Piece> getPiece(size_t index);
 
   virtual void completePiece(const SharedHandle<Piece>& piece);
 
-  virtual void cancelPiece(const SharedHandle<Piece>& piece);
+  virtual void cancelPiece(const SharedHandle<Piece>& piece, cuid_t cuid);
 
   virtual bool hasPiece(size_t index);
 
@@ -187,8 +222,16 @@ public:
 
   virtual bool isSelectiveDownloadingMode();
 
-  virtual bool isEndGame();
+  virtual bool isEndGame()
+  {
+    return endGame_;
+  }
   
+  virtual void enterEndGame()
+  {
+    endGame_ = true;
+  }
+
   virtual SharedHandle<DiskAdaptor> getDiskAdaptor();
 
   virtual size_t getPieceLength(size_t index);
@@ -257,4 +300,4 @@ typedef SharedHandle<DefaultPieceStorage> DefaultPieceStorageHandle;
 
 } // namespace aria2
 
-#endif // _D_DEFAULT_PIECE_STORAGE_H_
+#endif // D_DEFAULT_PIECE_STORAGE_H
