@@ -46,6 +46,8 @@
 #include "uri.h"
 #include "PeerStat.h"
 #include "fmt.h"
+#include "ServerStatMan.h"
+#include "ServerStat.h"
 
 namespace aria2 {
 
@@ -170,9 +172,23 @@ FileEntry::getRequest
         break;
       }
     }
-  } else {    
-    req = requestPool_.front();
-    requestPool_.pop_front();
+  } else {
+    // Skip Request object if it is still
+    // sleeping(Request::getWakeTime() < global::wallclock).  If all
+    // pooled objects are sleeping, return first one.  Caller should
+    // inspect returned object's getWakeTime().
+    std::deque<SharedHandle<Request> >::iterator i = requestPool_.begin();
+    std::deque<SharedHandle<Request> >::iterator eoi = requestPool_.end();
+    for(; i != eoi; ++i) {
+      if((*i)->getWakeTime() <= global::wallclock) {
+        break;
+      }
+    }
+    if(i == eoi) {
+      i = requestPool_.begin();
+    }
+    req = *i;
+    requestPool_.erase(i);
     inFlightRequests_.push_back(req);
     A2_LOG_DEBUG(fmt("Picked up from pool: %s", req->getUri().c_str()));
   }
@@ -201,12 +217,76 @@ FileEntry::findFasterRequest(const SharedHandle<Request>& base)
     SharedHandle<Request> fastestRequest = requestPool_.front();
     requestPool_.pop_front();
     inFlightRequests_.push_back(fastestRequest);
-    lastFasterReplace_.reset();
+    lastFasterReplace_ = global::wallclock;
     return fastestRequest;
   }
   return SharedHandle<Request>();
 }
 
+SharedHandle<Request>
+FileEntry::findFasterRequest
+(const SharedHandle<Request>& base,
+ const std::vector<std::pair<size_t, std::string> >& usedHosts,
+ const SharedHandle<ServerStatMan>& serverStatMan)
+{
+  const int startupIdleTime = 10;
+  const unsigned int SPEED_THRESHOLD = 20*1024;
+  if(lastFasterReplace_.difference(global::wallclock) < startupIdleTime) {
+    return SharedHandle<Request>();
+  }
+  std::vector<std::string> inFlightHosts;
+  enumerateInFlightHosts(inFlightRequests_.begin(), inFlightRequests_.end(),
+                         std::back_inserter(inFlightHosts));
+  const SharedHandle<PeerStat>& basestat = base->getPeerStat();
+  A2_LOG_DEBUG("Search faster server using ServerStat.");
+  // Use first 10 good URIs to introduce some randomness.
+  const size_t NUM_URI = 10;
+  std::vector<std::pair<SharedHandle<ServerStat>, std::string> > fastCands;
+  std::vector<std::string> normCands;
+  for(std::deque<std::string>::const_iterator i = uris_.begin(),
+        eoi = uris_.end(); i != eoi && fastCands.size() < NUM_URI; ++i) {
+    uri::UriStruct us;
+    if(!uri::parse(us, *i)) {
+      continue;
+    }
+    if(std::count(inFlightHosts.begin(), inFlightHosts.end(),us.host)
+       >= static_cast<int>(maxConnectionPerServer_)) {
+      A2_LOG_DEBUG(fmt("%s has already used %d times, not considered.",
+                       (*i).c_str(),
+                       static_cast<int>(maxConnectionPerServer_)));
+      continue;
+    }
+    if(findSecond(usedHosts.begin(), usedHosts.end(), us.host) !=
+       usedHosts.end()) {
+      A2_LOG_DEBUG(fmt("%s is in usedHosts, not considered", (*i).c_str()));
+      continue;
+    }
+    SharedHandle<ServerStat> ss = serverStatMan->find(us.host, us.protocol);
+    if(ss && ss->isOK()) {
+      if((basestat &&
+          ss->getDownloadSpeed() > basestat->calculateDownloadSpeed()*1.5) ||
+         (!basestat && ss->getDownloadSpeed() > SPEED_THRESHOLD)) {
+        fastCands.push_back(std::make_pair(ss, *i));
+      }
+    }
+  }
+  if(!fastCands.empty()) {
+    std::sort(fastCands.begin(), fastCands.end(), ServerStatFaster());
+    SharedHandle<Request> fastestRequest(new Request());
+    const std::string& uri = fastCands.front().second;
+    A2_LOG_DEBUG(fmt("Selected %s from fastCands", uri.c_str()));
+    fastestRequest->setUri(uri);
+    fastestRequest->setReferer(base->getReferer());
+    uris_.erase(std::find(uris_.begin(), uris_.end(), uri));
+    spentUris_.push_back(uri);
+    inFlightRequests_.push_back(fastestRequest);
+    lastFasterReplace_ = global::wallclock;
+    return fastestRequest;
+  }
+  A2_LOG_DEBUG("No faster server found.");
+  return SharedHandle<Request>();
+}
+
 namespace {
 class RequestFaster {
 public:
