@@ -52,20 +52,12 @@ const std::string HttpHeader::CONTENT_DISPOSITION("Content-Disposition");
 
 const std::string HttpHeader::SET_COOKIE("Set-Cookie");
 
-const std::string HttpHeader::CHUNKED("chunked");
-
-const std::string HttpHeader::GZIP("gzip");
-
-const std::string HttpHeader::DEFLATE("deflate");
-
 const std::string HttpHeader::CONTENT_TYPE("Content-Type");
 
 const std::string HttpHeader::RETRY_AFTER("Retry-After");
 
 const std::string HttpHeader::CONNECTION("Connection");
 
-const std::string HttpHeader::CLOSE("close");
-
 const std::string HttpHeader::CONTENT_LENGTH("Content-Length");
 
 const std::string HttpHeader::CONTENT_RANGE("Content-Range");
@@ -74,29 +66,18 @@ const std::string HttpHeader::LAST_MODIFIED("Last-Modified");
 
 const std::string HttpHeader::ACCEPT_ENCODING("Accept-Encoding");
 
-const std::string HttpHeader::HTTP_1_1("HTTP/1.1");
+const std::string HttpHeader::LINK("Link");
 
-const std::string HttpHeader::S200("200");
+const std::string HttpHeader::DIGEST("Digest");
 
-const std::string HttpHeader::S206("206");
+const char HttpHeader::HTTP_1_1[] = "HTTP/1.1";
+const char HttpHeader::CLOSE[] = "close";
+const char HttpHeader::CHUNKED[] = "chunked";
+const char HttpHeader::GZIP[] = "gzip";
+const char HttpHeader::DEFLATE[] = "deflate";
 
-const std::string HttpHeader::S300("300");
-
-const std::string HttpHeader::S301("301");
-
-const std::string HttpHeader::S302("302");
-
-const std::string HttpHeader::S303("303");
-
-const std::string HttpHeader::S304("304");
-
-const std::string HttpHeader::S307("307");
-
-const std::string HttpHeader::S400("400");
-
-const std::string HttpHeader::S401("401");
-  
-const std::string HttpHeader::S404("404");
+HttpHeader::HttpHeader() {}
+HttpHeader::~HttpHeader() {}
 
 void HttpHeader::put(const std::string& name, const std::string& value) {
   std::multimap<std::string, std::string>::value_type vt
@@ -133,6 +114,14 @@ std::vector<std::string> HttpHeader::get(const std::string& name) const
   return v;
 }
 
+std::pair<std::multimap<std::string, std::string>::const_iterator,
+          std::multimap<std::string, std::string>::const_iterator>
+HttpHeader::getIterator(const std::string& name) const
+{
+  std::string n(util::toLower(name));
+  return table_.equal_range(n);
+}
+
 unsigned int HttpHeader::getFirstAsUInt(const std::string& name) const {
   return getFirstAsULLInt(name);
 }
@@ -169,7 +158,7 @@ RangeHandle HttpHeader::getRange() const
     // but some server returns '100-199/100', omitting bytes-unit sepcifier
     // 'bytes'.
     std::pair<std::string, std::string> splist;
-    util::split(splist, rangeStr, ' ');
+    util::divide(splist, rangeStr, ' ');
     if(splist.second.empty()) {
       // we assume bytes-unit specifier omitted.
       byteRangeSpec = splist.first;
@@ -178,10 +167,18 @@ RangeHandle HttpHeader::getRange() const
     }
   }
   std::pair<std::string, std::string> byteRangeSpecPair;
-  util::split(byteRangeSpecPair, byteRangeSpec, '/');
+  util::divide(byteRangeSpecPair, byteRangeSpec, '/');
+
+  if(util::strip(byteRangeSpecPair.first) == "*" ||
+     util::strip(byteRangeSpecPair.second) == "*") {
+    // If byte-range-resp-spec or instance-length is "*", we returns
+    // empty Range. The former is usually sent with 416 (Request range
+    // not satisfiable) status.
+    return SharedHandle<Range>(new Range());
+  }
 
   std::pair<std::string, std::string> byteRangeRespSpecPair;
-  util::split(byteRangeRespSpecPair, byteRangeSpecPair.first, '-');
+  util::divide(byteRangeRespSpecPair, byteRangeSpecPair.first, '-');
 
   off_t startByte = util::parseLLInt(byteRangeRespSpecPair.first);
   off_t endByte = util::parseLLInt(byteRangeRespSpecPair.second);
@@ -190,11 +187,6 @@ RangeHandle HttpHeader::getRange() const
   return SharedHandle<Range>(new Range(startByte, endByte, entityLength));
 }
 
-void HttpHeader::setResponseStatus(const std::string& responseStatus)
-{
-  responseStatus_ = responseStatus;
-}
-
 void HttpHeader::setVersion(const std::string& version)
 {
   version_ = version;
@@ -213,14 +205,25 @@ void HttpHeader::setRequestPath(const std::string& requestPath)
 void HttpHeader::fill(std::istream& in)
 {
   std::string line;
-  while(std::getline(in, line)) {
-    line = util::trim(line);
+  std::getline(in, line);
+  while(in) {
+    line = util::strip(line);
     if(line.empty()) {
-      continue;
+      std::getline(in, line);
+    } else {
+      std::pair<std::string, std::string> hp;
+      util::divide(hp, line, ':');
+      while(std::getline(in, line)) {
+        if(!line.empty() && (line[0] == ' ' || line[0] == '\t')) {
+          line = util::strip(line);
+          hp.second += " ";
+          hp.second += line;
+        } else {
+          break;
+        }
+      }
+      put(hp.first, hp.second);
     }
-    std::pair<std::string, std::string> hp;
-    util::split(hp, line, ':');
-    put(hp.first, hp.second);
   }
 }
 
