@@ -44,65 +44,73 @@
 #include "message.h"
 #include "DlAbortEx.h"
 #include "DlRetryEx.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "A2STR.h"
 #include "CookieStorage.h"
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
 #include "ChunkedDecodingStreamFilter.h"
-#ifdef HAVE_LIBZ
+#include "error_code.h"
+#include "prefs.h"
+#include "Option.h"
+#include "Checksum.h"
+#include "uri.h"
+#include "MetalinkHttpEntry.h"
+#include "Base64.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+#include "MessageDigest.h"
+#endif // ENABLE_MESSAGE_DIGEST
+#ifdef HAVE_ZLIB
 # include "GZipDecodingStreamFilter.h"
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
 
 namespace aria2 {
 
-HttpResponse::HttpResponse():cuid_(0),
-                             logger_(LogFactory::getInstance())
+HttpResponse::HttpResponse()
+  : cuid_(0)
 {}
 
 HttpResponse::~HttpResponse() {}
 
 void HttpResponse::validateResponse() const
 {
-  const std::string& status = getResponseStatus();
-  if(status >= HttpHeader::S400) {
+  int statusCode = getStatusCode();
+  if(statusCode >= 400) {
     return;
   }
-  if(status == HttpHeader::S304) {
-    if(httpRequest_->getIfModifiedSinceHeader().empty()) {
-      throw DL_ABORT_EX("Got 304 without If-Modified-Since");
+  if(statusCode == 304) {
+    if(!httpRequest_->conditionalRequest()) {
+      throw DL_ABORT_EX2("Got 304 without If-Modified-Since or If-None-Match",
+                         error_code::HTTP_PROTOCOL_ERROR);
     }
-  } else if(status == HttpHeader::S301 ||
-     status == HttpHeader::S302 ||
-     status == HttpHeader::S303 ||
-     status == HttpHeader::S307) {
+  } else if(statusCode == 301 ||
+            statusCode == 302 ||
+            statusCode == 303 ||
+            statusCode == 307) {
     if(!httpHeader_->defined(HttpHeader::LOCATION)) {
-      throw DL_ABORT_EX
-        (StringFormat(EX_LOCATION_HEADER_REQUIRED,
-                      util::parseUInt(status)).str());
+      throw DL_ABORT_EX2(fmt(EX_LOCATION_HEADER_REQUIRED, statusCode),
+                         error_code::HTTP_PROTOCOL_ERROR);
     }
     return;
-  } else if(status == HttpHeader::S200 ||
-            status == HttpHeader::S206) {
+  } else if(statusCode == 200 || statusCode == 206) {
     if(!httpHeader_->defined(HttpHeader::TRANSFER_ENCODING)) {
       // compare the received range against the requested range
       RangeHandle responseRange = httpHeader_->getRange();
       if(!httpRequest_->isRangeSatisfied(responseRange)) {
         throw DL_ABORT_EX2
-          (StringFormat
-           (EX_INVALID_RANGE_HEADER,
-            util::itos(httpRequest_->getStartByte(), true).c_str(),
-            util::itos(httpRequest_->getEndByte(), true).c_str(),
-            util::uitos(httpRequest_->getEntityLength(), true).c_str(),
-            util::itos(responseRange->getStartByte(), true).c_str(),
-            util::itos(responseRange->getEndByte(), true).c_str(),
-            util::uitos(responseRange->getEntityLength(), true).c_str()).str(),
-           downloadresultcode::CANNOT_RESUME);
+          (fmt(EX_INVALID_RANGE_HEADER,
+               util::itos(httpRequest_->getStartByte(), true).c_str(),
+               util::itos(httpRequest_->getEndByte(), true).c_str(),
+               util::uitos(httpRequest_->getEntityLength(), true).c_str(),
+               util::itos(responseRange->getStartByte(), true).c_str(),
+               util::itos(responseRange->getEndByte(), true).c_str(),
+               util::uitos(responseRange->getEntityLength(), true).c_str()),
+           error_code::CANNOT_RESUME);
       }
     }
   } else {
-    throw DL_ABORT_EX
-      (StringFormat("Unexpected status %s", status.c_str()).str());
+    throw DL_ABORT_EX2(fmt("Unexpected status %d", statusCode),
+                       error_code::HTTP_PROTOCOL_ERROR);
   }
 }
 
@@ -119,10 +127,9 @@ std::string HttpResponse::determinFilename() const
       return file;
     }
   } else {
-    if(logger_->info()) {
-      logger_->info(MSG_CONTENT_DISPOSITION_DETECTED,
-                    util::itos(cuid_).c_str(), contentDisposition.c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_CONTENT_DISPOSITION_DETECTED,
+                    cuid_,
+                    contentDisposition.c_str()));
     return contentDisposition;
   }
 }
@@ -130,21 +137,23 @@ std::string HttpResponse::determinFilename() const
 void HttpResponse::retrieveCookie()
 {
   Time now;
-  std::vector<std::string> v = httpHeader_->get(HttpHeader::SET_COOKIE);
-  for(std::vector<std::string>::const_iterator itr = v.begin(), eoi = v.end();
-      itr != eoi; ++itr) {
+  std::pair<std::multimap<std::string, std::string>::const_iterator,
+            std::multimap<std::string, std::string>::const_iterator> r =
+    httpHeader_->getIterator(HttpHeader::SET_COOKIE);
+  for(; r.first != r.second; ++r.first) {
     httpRequest_->getCookieStorage()->parseAndStore
-      (*itr, httpRequest_->getHost(), httpRequest_->getDir(), now.getTime());
+      ((*r.first).second, httpRequest_->getHost(), httpRequest_->getDir(),
+       now.getTime());
   }
 }
 
 bool HttpResponse::isRedirect() const
 {
-  const std::string& status = getResponseStatus();
-  return (HttpHeader::S301 == status ||
-          HttpHeader::S302 == status ||
-          HttpHeader::S303 == status ||
-          HttpHeader::S307 == status) &&
+  int statusCode = getStatusCode();
+  return (301 == statusCode ||
+          302 == statusCode ||
+          303 == statusCode ||
+          307 == statusCode) &&
     httpHeader_->defined(HttpHeader::LOCATION);
 }
 
@@ -152,15 +161,14 @@ void HttpResponse::processRedirect()
 {
   
   if(httpRequest_->getRequest()->redirectUri(getRedirectURI())) {
-    if(logger_->info()) {
-      logger_->info(MSG_REDIRECT, util::itos(cuid_).c_str(),
-                    httpRequest_->getRequest()->getCurrentUri().c_str());
-    }
+    A2_LOG_INFO(fmt(MSG_REDIRECT,
+                    cuid_,
+                    httpRequest_->getRequest()->getCurrentUri().c_str()));
   } else {
     throw DL_RETRY_EX
-      (StringFormat("CUID#%s - Redirect to %s failed. It may not be a valid"
-                    " URI.", util::itos(cuid_).c_str(),
-                    httpRequest_->getRequest()->getCurrentUri().c_str()).str());
+      (fmt("CUID#%lld - Redirect to %s failed. It may not be a valid URI.",
+           cuid_,
+           httpRequest_->getRequest()->getCurrentUri().c_str()));
   }
 }
 
@@ -206,18 +214,18 @@ const std::string& HttpResponse::getContentEncoding() const
 SharedHandle<StreamFilter> HttpResponse::getContentEncodingStreamFilter() const
 {
   SharedHandle<StreamFilter> filter;
-#ifdef HAVE_LIBZ
+#ifdef HAVE_ZLIB
   if(getContentEncoding() == HttpHeader::GZIP ||
      getContentEncoding() == HttpHeader::DEFLATE) {
     filter.reset(new GZipDecodingStreamFilter());
   }
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
   return filter;
 }
 
 uint64_t HttpResponse::getContentLength() const
 {
-  if(httpHeader_.isNull()) {
+  if(!httpHeader_) {
     return 0;
   } else {
     return httpHeader_->getRange()->getContentLength();
@@ -226,7 +234,7 @@ uint64_t HttpResponse::getContentLength() const
 
 uint64_t HttpResponse::getEntityLength() const
 {
-  if(httpHeader_.isNull()) {
+  if(!httpHeader_) {
     return 0;
   } else {
     return httpHeader_->getRange()->getEntityLength();
@@ -235,7 +243,7 @@ uint64_t HttpResponse::getEntityLength() const
 
 std::string HttpResponse::getContentType() const
 {
-  if(httpHeader_.isNull()) {
+  if(!httpHeader_) {
     return A2STR::NIL;
   } else {
     std::pair<std::string, std::string> p;
@@ -254,10 +262,9 @@ void HttpResponse::setHttpRequest(const SharedHandle<HttpRequest>& httpRequest)
   httpRequest_ = httpRequest;
 }
 
-// TODO return std::string
-const std::string& HttpResponse::getResponseStatus() const
+int HttpResponse::getStatusCode() const
 {
-  return httpHeader_->getResponseStatus();
+  return httpHeader_->getStatusCode();
 }
 
 bool HttpResponse::hasRetryAfter() const
@@ -290,4 +297,147 @@ bool HttpResponse::supportsPersistentConnection() const
      != std::string::npos);
 }
 
+namespace {
+bool parseMetalinkHttpLink(MetalinkHttpEntry& result, const std::string& s)
+{
+  std::string::const_iterator first = std::find(s.begin(), s.end(), '<');
+  if(first == s.end()) {
+    return false;
+  }
+  std::string::const_iterator last = std::find(first, s.end(), '>');
+  if(last == s.end()) {
+    return false;
+  }
+  std::string uri = util::stripIter(first+1, last);
+  if(uri.empty()) {
+    return false;
+  } else {
+    result.uri = uri;
+  }
+  last = std::find(last, s.end(), ';');
+  if(last != s.end()) {
+    ++last;
+  }
+  bool ok = false;
+  while(1) {
+    std::string name, value;
+    std::pair<std::string::const_iterator, bool> r =
+      util::nextParam(name, value, last, s.end(), ';');
+    last = r.first;
+    if(!r.second) {
+      break;
+    }
+    if(value.empty()) {
+      if(name == "pref") {
+        result.pref = true;
+      }
+    } else {
+      if(name == "rel") {
+        if(value == "duplicate") {
+          ok = true;
+        } else {
+          ok = false;
+        }
+      } else if(name == "pri") {
+        int32_t priValue;
+        if(util::parseIntNoThrow(priValue, value)) {
+          if(1 <= priValue && priValue <= 999999) {
+            result.pri = priValue;
+          }
+        }
+      } else if(name == "geo") {
+        util::lowercase(value);
+        result.geo = value;
+      }
+    }
+  }
+  return ok;
+}
+} // namespace
+
+// Metalink/HTTP is defined by http://tools.ietf.org/html/rfc6249.
+// Link header field is defined by http://tools.ietf.org/html/rfc5988.
+void HttpResponse::getMetalinKHttpEntries
+(std::vector<MetalinkHttpEntry>& result,
+ const SharedHandle<Option>& option) const
+{
+  std::pair<std::multimap<std::string, std::string>::const_iterator,
+            std::multimap<std::string, std::string>::const_iterator> p =
+    httpHeader_->getIterator(HttpHeader::LINK);
+  for(; p.first != p.second; ++p.first) {
+    MetalinkHttpEntry e;
+    if(parseMetalinkHttpLink(e, (*p.first).second)) {
+      result.push_back(e);
+    }
+  }
+  if(!result.empty()) {
+    std::vector<std::string> locs;
+    if(option->defined(PREF_METALINK_LOCATION)) {
+      util::split(util::toLower(option->get(PREF_METALINK_LOCATION)),
+                  std::back_inserter(locs), ",", true);
+    }
+    for(std::vector<MetalinkHttpEntry>::iterator i = result.begin(),
+          eoi = result.end(); i != eoi; ++i) {
+      if(std::find(locs.begin(), locs.end(), (*i).geo) != locs.end()) {
+        (*i).pri -= 999999;
+      }
+    }
+  }
+  std::sort(result.begin(), result.end());
+}
+
+#ifdef ENABLE_MESSAGE_DIGEST
+// Digest header field is defined by
+// http://tools.ietf.org/html/rfc3230.
+void HttpResponse::getDigest(std::vector<Checksum>& result) const
+{
+  using std::swap;
+  std::pair<std::multimap<std::string, std::string>::const_iterator,
+            std::multimap<std::string, std::string>::const_iterator> p =
+    httpHeader_->getIterator(HttpHeader::DIGEST);
+  for(; p.first != p.second; ++p.first) {
+    const std::string& s = (*p.first).second;
+    std::string::const_iterator itr = s.begin();
+    while(1) {
+      std::string hashType, digest;
+      std::pair<std::string::const_iterator, bool> r =
+        util::nextParam(hashType, digest, itr, s.end(), ',');
+      itr = r.first;
+      if(!r.second) {
+        break;
+      }
+      util::lowercase(hashType);
+      if(!MessageDigest::supports(hashType)) {
+        continue;
+      }
+      std::string hexDigest = util::toHex(Base64::decode(digest));
+      if(!MessageDigest::isValidHash(hashType, hexDigest)) {
+        continue;
+      }
+      result.push_back(Checksum(hashType, hexDigest));
+    }
+  }
+  std::sort(result.begin(), result.end(), HashTypeStronger());
+  std::vector<Checksum> temp;
+  for(std::vector<Checksum>::iterator i = result.begin(),
+        eoi = result.end(); i != eoi;) {
+    bool ok = true;
+    std::vector<Checksum>::iterator j = i+1;
+    for(; j != eoi; ++j) {
+      if((*i).getHashType() != (*j).getHashType()) {
+        break;
+      }
+      if((*i).getDigest() != (*j).getDigest()) {
+        ok = false;
+      }
+    }
+    if(ok) {
+      temp.push_back(*i);
+    }
+    i = j;
+  }
+  swap(temp, result);
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
 } // namespace aria2
