@@ -38,6 +38,7 @@
 #include "common.h"
 
 #include <stdint.h>
+#include <vector>
 
 #include "SharedHandle.h"
 #include "TimeA2.h"
@@ -47,15 +48,16 @@ namespace aria2 {
 
 class HttpRequest;
 class HttpHeader;
-class Logger;
 class StreamFilter;
+class MetalinkHttpEntry;
+class Option;
+class Checksum;
 
 class HttpResponse {
 private:
   cuid_t cuid_;
   SharedHandle<HttpRequest> httpRequest_;
   SharedHandle<HttpHeader> httpHeader_;
-  Logger* logger_;
 public:
   HttpResponse();
   
@@ -108,7 +110,7 @@ public:
     return httpHeader_;
   }
 
-  const std::string& getResponseStatus() const;
+  int getStatusCode() const;
 
   void setHttpRequest(const SharedHandle<HttpRequest>& httpRequest);
 
@@ -129,6 +131,17 @@ public:
   Time getLastModifiedTime() const;
 
   bool supportsPersistentConnection() const;
+
+  void getMetalinKHttpEntries
+  (std::vector<MetalinkHttpEntry>& result,
+   const SharedHandle<Option>& option) const;
+#ifdef ENABLE_MESSAGE_DIGEST
+  // Returns all digests specified in Digest header field.  Sort
+  // strong algorithm first. Strength is defined in MessageDigest. If
+  // several same digest algorithms are available, but they have
+  // different value, they are all ignored.
+  void getDigest(std::vector<Checksum>& result) const;
+#endif // ENABLE_MESSAGE_DIGEST
 };
 
 } // namespace aria2
