@@ -57,7 +57,7 @@
 #include "Socket.h"
 #include "message.h"
 #include "prefs.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "HttpSkipResponseCommand.h"
 #include "HttpHeader.h"
 #include "LogFactory.h"
@@ -66,26 +66,70 @@
 #include "AuthConfig.h"
 #include "a2functional.h"
 #include "URISelector.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
 #include "CheckIntegrityEntry.h"
 #include "StreamFilter.h"
 #include "SinkStreamFilter.h"
 #include "ChunkedDecodingStreamFilter.h"
 #include "uri.h"
-#ifdef HAVE_LIBZ
+#include "SocketRecvBuffer.h"
+#include "MetalinkHttpEntry.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+#include "Checksum.h"
+#endif // ENABLE_MESSAGE_DIGEST
+#ifdef HAVE_ZLIB
 # include "GZipDecodingStreamFilter.h"
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
 
 namespace aria2 {
 
-static SharedHandle<StreamFilter> getTransferEncodingStreamFilter
+namespace {
+SharedHandle<StreamFilter> getTransferEncodingStreamFilter
 (const SharedHandle<HttpResponse>& httpResponse,
- const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>());
+ const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>())
+{
+  SharedHandle<StreamFilter> filter;
+  if(httpResponse->isTransferEncodingSpecified()) {
+    filter = httpResponse->getTransferEncodingStreamFilter();
+    if(!filter) {
+      throw DL_ABORT_EX
+        (fmt(EX_TRANSFER_ENCODING_NOT_SUPPORTED,
+             httpResponse->getTransferEncoding().c_str()));
+    }
+    filter->init();
+    filter->installDelegate(delegate);
+  }
+  if(!filter) {
+    filter = delegate;
+  }
+  return filter;
+}
+} // namespace
 
-static SharedHandle<StreamFilter> getContentEncodingStreamFilter
+namespace {
+SharedHandle<StreamFilter> getContentEncodingStreamFilter
 (const SharedHandle<HttpResponse>& httpResponse,
- const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>());
+ const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>())
+{
+  SharedHandle<StreamFilter> filter;
+  if(httpResponse->isContentEncodingSpecified()) {
+    filter = httpResponse->getContentEncodingStreamFilter();
+    if(!filter) {
+      A2_LOG_INFO
+        (fmt("Content-Encoding %s is specified, but the current implementation"
+             "doesn't support it. The decoding process is skipped and the"
+             "downloaded content will be still encoded.",
+             httpResponse->getContentEncoding().c_str()));
+    } else {
+      filter->init();
+      filter->installDelegate(delegate);
+    }
+  }
+  if(!filter) {
+    filter = delegate;
+  }
+  return filter;
+}
+} // namespace
 
 HttpResponseCommand::HttpResponseCommand
 (cuid_t cuid,
@@ -95,9 +139,12 @@ HttpResponseCommand::HttpResponseCommand
  const HttpConnectionHandle& httpConnection,
  DownloadEngine* e,
  const SocketHandle& s)
-  :AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
-   httpConnection_(httpConnection)
-{}
+  : AbstractCommand(cuid, req, fileEntry, requestGroup, e, s,
+                    httpConnection->getSocketRecvBuffer()),
+    httpConnection_(httpConnection)
+{
+  checkSocketRecvBuffer();
+}
 
 HttpResponseCommand::~HttpResponseCommand() {}
 
@@ -105,7 +152,7 @@ bool HttpResponseCommand::executeInternal()
 {
   SharedHandle<HttpRequest> httpRequest =httpConnection_->getFirstHttpRequest();
   SharedHandle<HttpResponse> httpResponse = httpConnection_->receiveResponse();
-  if(httpResponse.isNull()) {
+  if(!httpResponse) {
     // The server has not responded to our request yet.
     // For socket->wantRead() == true, setReadCheckSocket(socket) is already
     // done in the constructor.
@@ -126,31 +173,63 @@ bool HttpResponseCommand::executeInternal()
   if(getRequest()->isPipeliningEnabled()) {
     getRequest()->setMaxPipelinedRequest
       (getOption()->getAsInt(PREF_MAX_HTTP_PIPELINING));
+  } else {
+    getRequest()->setMaxPipelinedRequest(1);
   }
 
-  if(!httpResponse->getHttpRequest()->getIfModifiedSinceHeader().empty()) {
-    if(httpResponse->getResponseStatus() == HttpHeader::S304) {
-      uint64_t totalLength = httpResponse->getEntityLength();
-      getFileEntry()->setLength(totalLength);
-      getRequestGroup()->initPieceStorage();
-      getPieceStorage()->markAllPiecesDone();
-      // Just set checksum verification done.
-      getDownloadContext()->setChecksumVerified(true);
-      getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                          util::itos(getRequestGroup()->getGID()).c_str(),
-                          getRequestGroup()->getFirstFilePath().c_str());
-      poolConnection();
-      getFileEntry()->poolRequest(getRequest());
-      return true;
-    } else if(httpResponse->getResponseStatus() == HttpHeader::S200 ||
-              httpResponse->getResponseStatus() == HttpHeader::S206) {
-      // Remote file is newer than local file. We allow overwrite.
-      getOption()->put(PREF_ALLOW_OVERWRITE, A2_V_TRUE);
-    }
+  int statusCode = httpResponse->getStatusCode();
+
+  if(statusCode == 304) {
+    uint64_t totalLength = httpResponse->getEntityLength();
+    getFileEntry()->setLength(totalLength);
+    getRequestGroup()->initPieceStorage();
+    getPieceStorage()->markAllPiecesDone();
+    // Just set checksum verification done.
+    getDownloadContext()->setChecksumVerified(true);
+    A2_LOG_NOTICE(fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
+                      util::itos(getRequestGroup()->getGID()).c_str(),
+                      getRequestGroup()->getFirstFilePath().c_str()));
+    poolConnection();
+    getFileEntry()->poolRequest(getRequest());
+    return true;
   }
-  if(httpResponse->getResponseStatus() >= HttpHeader::S300 &&
-     httpResponse->getResponseStatus() != HttpHeader::S304) {
-    if(httpResponse->getResponseStatus() == HttpHeader::S404) {
+  if(!getPieceStorage()) {
+    // Metalink/HTTP
+    if(!getDownloadContext()->getMetalinkServerContacted()) {
+      if(httpHeader->defined(HttpHeader::LINK)) {
+        getDownloadContext()->setMetalinkServerContacted(true);
+        std::vector<MetalinkHttpEntry> entries;
+        httpResponse->getMetalinKHttpEntries(entries, getOption());
+        for(std::vector<MetalinkHttpEntry>::iterator i = entries.begin(),
+              eoi = entries.end(); i != eoi; ++i) {
+          getFileEntry()->addUri((*i).uri);
+          A2_LOG_DEBUG(fmt("Adding URI=%s", (*i).uri.c_str()));
+        }
+      }
+    }
+#ifdef ENABLE_MESSAGE_DIGEST
+    if(httpHeader->defined(HttpHeader::DIGEST)) {
+      std::vector<Checksum> checksums;
+      httpResponse->getDigest(checksums);
+      for(std::vector<Checksum>::iterator i = checksums.begin(),
+            eoi = checksums.end(); i != eoi; ++i) {
+        if(getDownloadContext()->getHashType().empty()) {
+          A2_LOG_DEBUG(fmt("Setting digest: type=%s, digest=%s",
+                           (*i).getHashType().c_str(),
+                           (*i).getDigest().c_str()));
+          getDownloadContext()->setDigest((*i).getHashType(), (*i).getDigest());
+          break;
+        } else {
+          if(checkChecksum(getDownloadContext(), *i)) {
+            break;
+          }
+        }
+      }
+    }
+#endif // ENABLE_MESSAGE_DIGEST
+  }
+  if(statusCode >= 300) {
+    if(statusCode == 404) {
       getRequestGroup()->increaseAndValidateFileNotFoundCount();
     }
     return skipResponseBody(httpResponse);
@@ -163,21 +242,23 @@ bool HttpResponseCommand::executeInternal()
       getFileEntry()->removeURIWhoseHostnameIs(us.host);
     }
   }
-  if(getPieceStorage().isNull()) {
+  if(!getPieceStorage()) {
+    util::removeMetalinkContentTypes(getRequestGroup());
     uint64_t totalLength = httpResponse->getEntityLength();
     getFileEntry()->setLength(totalLength);
     if(getFileEntry()->getPath().empty()) {
       getFileEntry()->setPath
         (util::createSafePath
-         (getDownloadContext()->getDir(), httpResponse->determinFilename()));
+         (getOption()->get(PREF_DIR), httpResponse->determinFilename()));
     }
     getFileEntry()->setContentType(httpResponse->getContentType());
     getRequestGroup()->preDownloadProcessing();
     if(getDownloadEngine()->getRequestGroupMan()->
        isSameFileBeingDownloaded(getRequestGroup())) {
-      throw DOWNLOAD_FAILURE_EXCEPTION
-        (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
-                      getRequestGroup()->getFirstFilePath().c_str()).str());
+      throw DOWNLOAD_FAILURE_EXCEPTION2
+        (fmt(EX_DUPLICATE_FILE_DOWNLOAD,
+             getRequestGroup()->getFirstFilePath().c_str()),
+         error_code::DUPLICATE_DOWNLOAD);
     }
     // update last modified time
     updateLastModifiedTime(httpResponse->getLastModifiedTime());
@@ -199,6 +280,19 @@ bool HttpResponseCommand::executeInternal()
       return handleDefaultEncoding(httpResponse);
     }
   } else {
+#ifdef ENABLE_MESSAGE_DIGEST
+    if(!getDownloadContext()->getHashType().empty() &&
+       httpHeader->defined(HttpHeader::DIGEST)) {
+      std::vector<Checksum> checksums;
+      httpResponse->getDigest(checksums);
+      for(std::vector<Checksum>::iterator i = checksums.begin(),
+            eoi = checksums.end(); i != eoi; ++i) {
+        if(checkChecksum(getDownloadContext(), *i)) {
+          break;
+        }
+      }
+    }
+#endif // ENABLE_MESSAGE_DIGEST
     // validate totalsize
     getRequestGroup()->validateTotalLength(getFileEntry()->getLength(),
                                        httpResponse->getEntityLength());
@@ -252,11 +346,10 @@ bool HttpResponseCommand::handleDefaultEncoding
 (const SharedHandle<HttpResponse>& httpResponse)
 {
   SharedHandle<HttpRequest> httpRequest = httpResponse->getHttpRequest();
-  getRequestGroup()->adjustFilename
-    (SharedHandle<BtProgressInfoFile>(new DefaultBtProgressInfoFile
-                                      (getDownloadContext(),
-                                       SharedHandle<PieceStorage>(),
-                                       getOption().get())));
+  SharedHandle<BtProgressInfoFile> progressInfoFile
+    (new DefaultBtProgressInfoFile
+     (getDownloadContext(), SharedHandle<PieceStorage>(), getOption().get()));
+  getRequestGroup()->adjustFilename(progressInfoFile);
   getRequestGroup()->initPieceStorage();
 
   if(getOption()->getAsBool(PREF_DRY_RUN)) {
@@ -266,7 +359,7 @@ bool HttpResponseCommand::handleDefaultEncoding
 
   SharedHandle<CheckIntegrityEntry> checkEntry =
     getRequestGroup()->createCheckIntegrityEntry();
-  if(checkEntry.isNull()) {
+  if(!checkEntry) {
     return true;
   }
   File file(getRequestGroup()->getFirstFilePath());
@@ -282,7 +375,7 @@ bool HttpResponseCommand::handleDefaultEncoding
   // Therefore, we shutdown the socket here if pipelining is enabled.
   DownloadCommand* command = 0;
   if(getRequest()->getMethod() == Request::METHOD_GET &&
-     !segment.isNull() && segment->getPositionToWrite() == 0 &&
+     segment && segment->getPositionToWrite() == 0 &&
      !getRequest()->isPipeliningEnabled()) {
     command = createHttpDownloadCommand
       (httpResponse,
@@ -305,51 +398,6 @@ bool HttpResponseCommand::handleDefaultEncoding
   return true;
 }
 
-static SharedHandle<StreamFilter> getTransferEncodingStreamFilter
-(const SharedHandle<HttpResponse>& httpResponse,
- const SharedHandle<StreamFilter>& delegate)
-{
-  SharedHandle<StreamFilter> filter;
-  if(httpResponse->isTransferEncodingSpecified()) {
-    filter = httpResponse->getTransferEncodingStreamFilter();
-    if(filter.isNull()) {
-      throw DL_ABORT_EX
-        (StringFormat(EX_TRANSFER_ENCODING_NOT_SUPPORTED,
-                      httpResponse->getTransferEncoding().c_str()).str());
-    }
-    filter->init();
-    filter->installDelegate(delegate);
-  }
-  if(filter.isNull()) {
-    filter = delegate;
-  }
-  return filter;
-}
-
-static SharedHandle<StreamFilter> getContentEncodingStreamFilter
-(const SharedHandle<HttpResponse>& httpResponse,
- const SharedHandle<StreamFilter>& delegate)
-{
-  SharedHandle<StreamFilter> filter;
-  if(httpResponse->isContentEncodingSpecified()) {
-    filter = httpResponse->getContentEncodingStreamFilter();
-    if(filter.isNull()) {
-      LogFactory::getInstance()->info
-        ("Content-Encoding %s is specified, but the current implementation"
-         "doesn't support it. The decoding process is skipped and the"
-         "downloaded content will be still encoded.",
-         httpResponse->getContentEncoding().c_str());
-    } else {
-      filter->init();
-      filter->installDelegate(delegate);
-    }
-  }
-  if(filter.isNull()) {
-    filter = delegate;
-  }
-  return filter;
-}
-
 bool HttpResponseCommand::handleOtherEncoding
 (const SharedHandle<HttpResponse>& httpResponse) {
   // We assume that RequestGroup::getTotalLength() == 0 here
@@ -376,7 +424,7 @@ bool HttpResponseCommand::handleOtherEncoding
      getContentEncodingStreamFilter(httpResponse));
   // If chunked transfer-encoding is specified, we have to read end of
   // chunk markers(0\r\n\r\n, for example).
-  bool chunkedUsed = !streamFilter.isNull() &&
+  bool chunkedUsed = streamFilter &&
     streamFilter->getName() == ChunkedDecodingStreamFilter::NAME;
 
   // For zero-length file, check existing file comparing its size
@@ -390,9 +438,9 @@ bool HttpResponseCommand::handleOtherEncoding
     getRequestGroup()->initPieceStorage();
     getPieceStorage()->markAllPiecesDone();
     getDownloadContext()->setChecksumVerified(true);
-    getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                        util::itos(getRequestGroup()->getGID()).c_str(),
-                        getRequestGroup()->getFirstFilePath().c_str());
+    A2_LOG_NOTICE(fmt(MSG_DOWNLOAD_ALREADY_COMPLETED,
+                      util::itos(getRequestGroup()->getGID()).c_str(),
+                      getRequestGroup()->getFirstFilePath().c_str()));
     poolConnection();
     return true;
   }
@@ -448,11 +496,12 @@ bool HttpResponseCommand::skipResponseBody
   return true;
 }
 
-static bool decideFileAllocation
+namespace {
+bool decideFileAllocation
 (const SharedHandle<StreamFilter>& filter)
 {
-#ifdef HAVE_LIBZ
-  for(SharedHandle<StreamFilter> f = filter; !f.isNull(); f = f->getDelegate()){
+#ifdef HAVE_ZLIB
+  for(SharedHandle<StreamFilter> f = filter; f; f = f->getDelegate()){
     // Since the compressed file's length are returned in the response header
     // and the decompressed file size is unknown at this point, disable file
     // allocation here.
@@ -460,9 +509,10 @@ static bool decideFileAllocation
       return false;
     }
   }
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
   return true;
 }
+} // namespace
 
 HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand
 (const SharedHandle<HttpResponse>& httpResponse,
@@ -503,4 +553,21 @@ void HttpResponseCommand::onDryRunFileFound()
   poolConnection();
 }
 
+#ifdef ENABLE_MESSAGE_DIGEST
+bool HttpResponseCommand::checkChecksum
+(const SharedHandle<DownloadContext>& dctx,
+ const Checksum& checksum)
+{
+  if(dctx->getHashType() == checksum.getHashType()) {
+    if(dctx->getDigest() == checksum.getDigest()) {
+      A2_LOG_INFO("Valid hash found in Digest header field.");
+      return true;
+    } else {
+      throw DL_ABORT_EX("Invalid hash found in Digest header field.");
+    }
+  }
+  return false;
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
 } // namespace aria2
