@@ -216,18 +216,20 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
 #endif // ENABLE_ASYNC_DNS
-#ifdef ENABLE_DIRECT_IO
   {
-    SharedHandle<OptionHandler> op(new BooleanOptionHandler
-                                   (PREF_ENABLE_DIRECT_IO,
-                                    TEXT_ENABLE_DIRECT_IO,
-                                    A2_V_TRUE,
-                                    OptionHandler::OPT_ARG));
+    // TODO Deprecated
+    SharedHandle<OptionHandler> op
+      (new DeprecatedOptionHandler
+       (SharedHandle<OptionHandler>(new BooleanOptionHandler
+                                    (PREF_ENABLE_DIRECT_IO,
+                                     TEXT_ENABLE_DIRECT_IO,
+                                     NO_DEFAULT_VALUE,
+                                     OptionHandler::OPT_ARG))));
+    op->addTag(TAG_DEPRECATED);
     op->addTag(TAG_ADVANCED);
     op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
-#endif // ENABLE_DIRECT_IO
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_ENABLE_RPC,
@@ -237,19 +239,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_RPC);
     handlers.push_back(op);
   }
-  {
-    // TODO Deprecated
-    SharedHandle<OptionHandler> op
-      (new DeprecatedOptionHandler
-       (SharedHandle<OptionHandler>(new BooleanOptionHandler
-                                    (PREF_ENABLE_XML_RPC,
-                                     TEXT_ENABLE_XML_RPC,
-                                     NO_DEFAULT_VALUE,
-                                     OptionHandler::OPT_ARG)),
-        PREF_ENABLE_RPC));
-    op->addTag(TAG_RPC);
-    handlers.push_back(op);
-  }
   {
     std::string params[] = {
 #ifdef HAVE_EPOLL
@@ -514,6 +503,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+                                   (PREF_PAUSE,
+                                    TEXT_PAUSE,
+                                    A2_V_FALSE,
+                                    OptionHandler::OPT_ARG));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_RPC);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_QUIET,
@@ -587,6 +586,15 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+                                   (PREF_TRUNCATE_CONSOLE_READOUT,
+                                    TEXT_TRUNCATE_CONSOLE_READOUT,
+                                    A2_V_TRUE,
+                                    OptionHandler::OPT_ARG));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
                                    (PREF_RPC_LISTEN_ALL,
@@ -630,69 +638,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->setEraseAfterParse(true);
     handlers.push_back(op);
   }
-  {
-    // TODO Deprecated
-    SharedHandle<OptionHandler> op
-      (new DeprecatedOptionHandler
-       (SharedHandle<OptionHandler>(new BooleanOptionHandler
-                                    (PREF_XML_RPC_LISTEN_ALL,
-                                     TEXT_XML_RPC_LISTEN_ALL,
-                                     NO_DEFAULT_VALUE,
-                                     OptionHandler::OPT_ARG)),
-        PREF_RPC_LISTEN_ALL));
-    op->addTag(TAG_RPC);
-    handlers.push_back(op);
-  }
-  {
-    // TODO Deprecated
-    SharedHandle<OptionHandler> op
-      (new DeprecatedOptionHandler
-       (SharedHandle<OptionHandler>(new NumberOptionHandler
-                                    (PREF_XML_RPC_LISTEN_PORT,
-                                     TEXT_XML_RPC_LISTEN_PORT,
-                                     NO_DEFAULT_VALUE,
-                                     1024, UINT16_MAX)),
-        PREF_RPC_LISTEN_PORT));
-    op->addTag(TAG_RPC);
-    handlers.push_back(op);
-  }
-  {
-    // TODO Deprecated
-     SharedHandle<OptionHandler> op
-       (new DeprecatedOptionHandler
-        (SharedHandle<OptionHandler>(new UnitNumberOptionHandler
-                                     (PREF_XML_RPC_MAX_REQUEST_SIZE,
-                                      TEXT_XML_RPC_MAX_REQUEST_SIZE,
-                                      NO_DEFAULT_VALUE,
-                                      0)),
-         PREF_RPC_MAX_REQUEST_SIZE));
-     op->addTag(TAG_RPC);
-     handlers.push_back(op);
-  }
-  {
-    // TODO Deprecated
-    SharedHandle<OptionHandler> op
-      (new DeprecatedOptionHandler
-       (SharedHandle<OptionHandler>(new DefaultOptionHandler
-                                    (PREF_XML_RPC_USER,
-                                     TEXT_XML_RPC_USER)),
-        PREF_RPC_USER));
-    op->addTag(TAG_RPC);
-    op->setEraseAfterParse(true);
-    handlers.push_back(op);
-  }
-  {
-    // TODO Deprecated
-    SharedHandle<OptionHandler> op
-      (new DeprecatedOptionHandler
-       (SharedHandle<OptionHandler>(new DefaultOptionHandler
-                                    (PREF_XML_RPC_PASSWD,
-                                     TEXT_XML_RPC_PASSWD)),
-        PREF_RPC_PASSWD));
-    op->addTag(TAG_RPC);
-    op->setEraseAfterParse(true);
-    handlers.push_back(op);
-  }                         
   // HTTP/FTP options
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
@@ -861,6 +806,17 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->hide();
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new ParameterOptionHandler
+                                   (PREF_STREAM_PIECE_SELECTOR,
+                                    TEXT_STREAM_PIECE_SELECTOR,
+                                    A2_V_DEFAULT,
+                                    A2_V_DEFAULT,
+                                    V_INORDER));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
                                    (PREF_TIMEOUT,
@@ -1007,6 +963,13 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->hide();
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+                                   (PREF_METALINK_LOCATION,
+                                    TEXT_METALINK_LOCATION));
+    op->addTag(TAG_METALINK);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_PRIVATE_KEY,
@@ -1727,13 +1690,6 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
-  {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
-                                   (PREF_METALINK_LOCATION,
-                                    TEXT_METALINK_LOCATION));
-    op->addTag(TAG_METALINK);
-    handlers.push_back(op);
-  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
                                    (PREF_METALINK_OS,
@@ -1797,6 +1753,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
       TAG_FILE,
       TAG_RPC,
       TAG_EXPERIMENTAL,
+      TAG_DEPRECATED,
       TAG_HELP,
       TAG_ALL
     };
