@@ -55,6 +55,7 @@
 #include "File.h"
 #include "FileEntry.h"
 #include "a2io.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
@@ -114,6 +115,13 @@ char NullOptionHandler::getShortName() const
   return 0;
 }
 
+bool NullOptionHandler::getEraseAfterParse() const
+{
+  return false;
+}
+
+void NullOptionHandler::setEraseAfterParse(bool eraseAfterParse) {}
+
 BooleanOptionHandler::BooleanOptionHandler
 (const std::string& optName,
  const std::string& description,
@@ -628,10 +636,13 @@ void HttpProxyOptionHandler::parseArg(Option& option, const std::string& optarg)
   } else {
     Request req;
     std::string uri;
-    if(util::startsWith(optarg, "http://")) {
+    if(util::startsWith(optarg, "http://") ||
+       util::startsWith(optarg, "https://") ||
+       util::startsWith(optarg, "ftp://")) {
       uri = optarg;
     } else {
-      uri = "http://"+optarg;
+      uri = "http://";
+      uri += optarg;
     }
     if(!req.setUri(uri)) {
       throw DL_ABORT_EX(_("unrecognized proxy format"));
@@ -725,4 +736,103 @@ std::string PrioritizePieceOptionHandler::createPossibleValuesString() const
   return "head[=SIZE], tail[=SIZE]";
 }
 
+DeprecatedOptionHandler::DeprecatedOptionHandler
+(const SharedHandle<OptionHandler>& depOptHandler,
+ const SharedHandle<OptionHandler>& repOptHandler)
+  : depOptHandler_(depOptHandler), repOptHandler_(repOptHandler)
+{}
+
+bool DeprecatedOptionHandler::canHandle(const std::string& optName)
+{
+  return depOptHandler_->canHandle(optName);
+}
+
+void DeprecatedOptionHandler::parse(Option& option, const std::string& arg)
+{
+  if(repOptHandler_) {
+    A2_LOG_WARN(fmt(_("--%s option is deprecated. Use --%s option instead."),
+                    depOptHandler_->getName().c_str(),
+                    repOptHandler_->getName().c_str()));
+    repOptHandler_->parse(option, arg);
+  } else {
+    A2_LOG_WARN(fmt(_("--%s option is deprecated."),
+                    depOptHandler_->getName().c_str()));
+  }
+}
+
+std::string DeprecatedOptionHandler::createPossibleValuesString() const
+{
+  return depOptHandler_->createPossibleValuesString();
+}
+
+bool DeprecatedOptionHandler::hasTag(const std::string& tag) const
+{
+  return depOptHandler_->hasTag(tag);
+}
+
+void DeprecatedOptionHandler::addTag(const std::string& tag)
+{
+  depOptHandler_->addTag(tag);
+}
+
+std::string DeprecatedOptionHandler::toTagString() const
+{
+  return depOptHandler_->toTagString();
+}
+
+const std::string& DeprecatedOptionHandler::getName() const
+{
+  return depOptHandler_->getName();
+}
+
+const std::string& DeprecatedOptionHandler::getDescription() const
+{
+  return depOptHandler_->getDescription();
+}
+
+const std::string& DeprecatedOptionHandler::getDefaultValue() const
+{
+  return depOptHandler_->getDefaultValue();
+}
+
+bool DeprecatedOptionHandler::isHidden() const
+{
+  return depOptHandler_->isHidden();
+}
+
+void DeprecatedOptionHandler::hide()
+{
+  depOptHandler_->hide();
+}
+
+OptionHandler::ARG_TYPE DeprecatedOptionHandler::getArgType() const
+{
+  return depOptHandler_->getArgType();
+}
+
+char DeprecatedOptionHandler::getShortName() const
+{
+  return depOptHandler_->getShortName();
+}
+
+int DeprecatedOptionHandler::getOptionID() const
+{
+  return depOptHandler_->getOptionID();
+}
+
+void DeprecatedOptionHandler::setOptionID(int id)
+{
+  depOptHandler_->setOptionID(id);
+}
+
+bool DeprecatedOptionHandler::getEraseAfterParse() const
+{
+  return depOptHandler_->getEraseAfterParse();
+}
+
+void DeprecatedOptionHandler::setEraseAfterParse(bool eraseAfterParse)
+{
+  depOptHandler_->setEraseAfterParse(eraseAfterParse);
+}
+
 } // namespace aria2
