@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_PEER_LISTEN_COMMAND_H_
-#define _D_PEER_LISTEN_COMMAND_H_
+#ifndef D_PEER_LISTEN_COMMAND_H
+#define D_PEER_LISTEN_COMMAND_H
 
 #include "Command.h"
 #include "SharedHandle.h"
@@ -47,15 +47,10 @@ class SocketCore;
 class PeerListenCommand : public Command {
 private:
   DownloadEngine* e_;
+  int family_;
   SharedHandle<SocketCore> socket_;
-  unsigned int lowestSpeedLimit_;
-
-  static unsigned int numInstance_;
-
-  static PeerListenCommand* instance_;
-
 public:
-  PeerListenCommand(cuid_t cuid, DownloadEngine* e);
+  PeerListenCommand(cuid_t cuid, DownloadEngine* e, int family);
 
   virtual ~PeerListenCommand();
   
@@ -69,21 +64,8 @@ public:
 
   // Returns binded port
   uint16_t getPort() const;
-
-  void setLowestSpeedLimit(unsigned int speed)
-  {
-    lowestSpeedLimit_ = speed;
-  }
-
-  static PeerListenCommand* getInstance(DownloadEngine* e);
-
-  static unsigned int getNumInstance()
-  {
-    return numInstance_;
-  }
-  
 };
 
 } // namespace aria2 
 
-#endif // _D_PEER_LISTEN_COMMAND_H_
+#endif // D_PEER_LISTEN_COMMAND_H
