@@ -71,8 +71,8 @@ private:
 
   unsigned int maxOverallUploadSpeedLimit_;
 
-  // truf if XML-RPC is enabled.
-  bool xmlRpc_;
+  // true if JSON-RPC/XML-RPC is enabled.
+  bool rpc_;
 
   bool queueCheck_;
 
@@ -132,14 +132,14 @@ public:
     return requestGroups_;
   }
 
-  SharedHandle<RequestGroup> findRequestGroup(gid_t gid) const;
+  SharedHandle<RequestGroup> findRequestGroup(a2_gid_t gid) const;
 
   const std::deque<SharedHandle<RequestGroup> >& getReservedGroups() const
   {
     return reservedGroups_;
   }
 
-  SharedHandle<RequestGroup> findReservedGroup(gid_t gid) const;
+  SharedHandle<RequestGroup> findReservedGroup(a2_gid_t gid) const;
 
   enum HOW {
     POS_SET,
@@ -156,9 +156,9 @@ public:
   // beyond the end of the queue, it moves the download to the
   // beginning or the end of the queue respectively.  Returns the
   // destination position.
-  size_t changeReservedGroupPosition(gid_t gid, int pos, HOW how);
+  size_t changeReservedGroupPosition(a2_gid_t gid, int pos, HOW how);
 
-  bool removeReservedGroup(gid_t gid);
+  bool removeReservedGroup(a2_gid_t gid);
 
   void showDownloadResults(std::ostream& o) const;
 
@@ -171,18 +171,21 @@ public:
     size_t completed_;
     size_t error_;
     size_t inProgress_;
+    size_t removed_;
     size_t waiting_;
     error_code::Value lastErrorResult_;
   public:
     DownloadStat(size_t completed,
                  size_t error,
                  size_t inProgress,
+                 size_t removed,
                  size_t waiting,
                  error_code::Value lastErrorResult =
                  error_code::FINISHED):
       completed_(completed),
       error_(error),
       inProgress_(inProgress),
+      removed_(removed),
       waiting_(waiting),
       lastErrorResult_(lastErrorResult) {}
 
@@ -209,11 +212,15 @@ public:
     return downloadResults_;
   }
 
-  SharedHandle<DownloadResult> findDownloadResult(gid_t gid) const;
+  SharedHandle<DownloadResult> findDownloadResult(a2_gid_t gid) const;
 
   // Removes all download results.
   void purgeDownloadResult();
 
+  // Removes download result of given gid. Returns true if download
+  // result was removed. Otherwise returns false.
+  bool removeDownloadResult(a2_gid_t gid);
+
   void addDownloadResult(const SharedHandle<DownloadResult>& downloadResult);
 
   SharedHandle<ServerStat> findServerStat(const std::string& hostname,
@@ -287,6 +294,11 @@ public:
 
   // Returns currently used hosts and its use count.
   void getUsedHosts(std::vector<std::pair<size_t, std::string> >& usedHosts);
+
+  const SharedHandle<ServerStatMan>& getServerStatMan() const
+  {
+    return serverStatMan_;
+  }
 };
 
 typedef SharedHandle<RequestGroupMan> RequestGroupManHandle;
