@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_UNKNOWN_LENGTH_PIECE_STORAGE_H_
-#define _D_UNKNOWN_LENGTH_PIECE_STORAGE_H_
+#ifndef D_UNKNOWN_LENGTH_PIECE_STORAGE_H
+#define D_UNKNOWN_LENGTH_PIECE_STORAGE_H
 
 #include "PieceStorage.h"
 
@@ -73,29 +73,40 @@ public:
    */
   virtual bool hasMissingPiece(const SharedHandle<Peer>& peer);
 
-  /**
-   * Returns a piece that the peer has but localhost doesn't.
-   * The piece will be marked "used" status in order to prevent other command
-   * from get the same piece. But in end game mode, same piece may be returned
-   * to several commands.
-   */
-  virtual SharedHandle<Piece> getMissingPiece(const SharedHandle<Peer>& peer);
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid);
+
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid);
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
 
   virtual SharedHandle<Piece> getMissingPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes);
-
-  /**
-   * Returns a piece that the peer has but localhost doesn't.
-   * Only pieces that declared as "fast" are returned.
-   * The piece will be marked "used" status in order to prevent other command
-   * from get the same piece. But in end game mode, same piece may be returned
-   * to several commands.
-   */
-  virtual SharedHandle<Piece> getMissingFastPiece(const SharedHandle<Peer>& peer);
-
-  virtual SharedHandle<Piece> getMissingFastPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes);
+  (const SharedHandle<Peer>& peer,
+   cuid_t cuid);
 
+  virtual SharedHandle<Piece> getMissingPiece
+  (const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid);
 #endif // ENABLE_BITTORRENT
 
   virtual bool hasMissingUnusedPiece();
@@ -103,8 +114,11 @@ public:
   /**
    * Returns a missing piece if available. Otherwise returns 0;
    */
-  virtual SharedHandle<Piece> getSparseMissingUnusedPiece
-  (size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length);
+  virtual SharedHandle<Piece> getMissingPiece
+  (size_t minSplitSize,
+   const unsigned char* ignoreBitfield,
+   size_t length,
+   cuid_t cuid);
 
   /**
    * Returns a missing piece whose index is index.
@@ -112,7 +126,7 @@ public:
    * then returns 0.
    * Also returns 0 if any of missing piece is not available.
    */
-  virtual SharedHandle<Piece> getMissingPiece(size_t index);
+  virtual SharedHandle<Piece> getMissingPiece(size_t index, cuid_t cuid);
 
   /**
    * Returns the piece denoted by index.
@@ -128,7 +142,7 @@ public:
   /**
    * Tells that the download of the specified piece is canceled.
    */
-  virtual void cancelPiece(const SharedHandle<Piece>& piece);
+  virtual void cancelPiece(const SharedHandle<Piece>& piece, cuid_t cuid);
 
   /**
    * Returns true if the specified piece is already downloaded.
@@ -211,6 +225,8 @@ public:
     return false;
   }
 
+  virtual void enterEndGame() {}
+
   virtual void setEndGamePieceNum(size_t num) {}
 
   virtual SharedHandle<DiskAdaptor> getDiskAdaptor();
@@ -282,4 +298,4 @@ typedef SharedHandle<UnknownLengthPieceStorage> UnknownLengthPieceStorageHandle;
 
 } // namespace aria2
 
-#endif // _D_UNKNOWN_LENGTH_PIECE_STORAGE_H_
+#endif // D_UNKNOWN_LENGTH_PIECE_STORAGE_H
