@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_XML2_SAX_METALINK_PROCESSOR_H_
-#define _D_XML2_SAX_METALINK_PROCESSOR_H_
+#ifndef D_XML2_SAX_METALINK_PROCESSOR_H
+#define D_XML2_SAX_METALINK_PROCESSOR_H
 
 #include "common.h"
 
@@ -43,6 +43,7 @@
 #include <libxml/xpath.h>
 
 #include "SharedHandle.h"
+#include "A2STR.h"
 
 namespace aria2 {
 
@@ -54,12 +55,19 @@ class MetalinkProcessor {
 private:
   SharedHandle<MetalinkParserStateMachine> stm_;
 public:
-  SharedHandle<Metalinker> parseFile(const std::string& filename);
+  MetalinkProcessor();
+
+  ~MetalinkProcessor();
+
+  SharedHandle<Metalinker> parseFile
+  (const std::string& filename,
+   const std::string& baseUri = A2STR::NIL);
 
   SharedHandle<Metalinker> parseFromBinaryStream
-  (const SharedHandle<BinaryStream>& binaryStream);
+  (const SharedHandle<BinaryStream>& binaryStream,
+   const std::string& baseUri = A2STR::NIL);
 };
 
 } // namespace aria2
 
-#endif // _D_XML2_SAX_METALINK_PROCESSOR_H_
+#endif // D_XML2_SAX_METALINK_PROCESSOR_H
