@@ -31,14 +31,14 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_A2NETCOMPAT_H_
-#define _D_A2NETCOMPAT_H_
+#ifndef D_A2NETCOMPAT_H
+#define D_A2NETCOMPAT_H
 
 #include "a2io.h"
 
 #ifdef __MINGW32__
 # ifndef WINVER
-#  define WINVER 0x501
+#  define WINVER 0x501u
 # endif // !WINVER
 # ifdef HAVE_WINSOCK2_H
 #  ifndef FD_SETSIZE
@@ -100,12 +100,18 @@
 # define sock_t int
 #endif
 
-#ifdef AI_ADDRCONFIG
-# define DEFAULT_AI_FLAGS AI_ADDRCONFIG
-#else // !AI_ADDRCONFIG
-# define DEFAULT_AI_FLAGS 0
+#ifndef AI_ADDRCONFIG
+# define AI_ADDRCONFIG 0
 #endif // !AI_ADDRCONFIG
 
+#define DEFAULT_AI_FLAGS AI_ADDRCONFIG
+
+#ifdef __MINGW32__
+# ifndef SHUT_WR
+#  define SHUT_WR SD_SEND
+# endif // !SHUT_WR
+#endif // __MINGW32__
+
 #ifdef __MINGW32__
 template<typename T>
 class wsaapi_auto_delete {
@@ -126,4 +132,4 @@ public:
 # define WSAAPI_AUTO_DELETE auto_delete
 #endif // !__MINGW32__
 
-#endif // _D_A2NETCOMPAT_H_
+#endif // D_A2NETCOMPAT_H
