@@ -41,9 +41,11 @@ namespace aria2 {
  */
 const std::string A2_V_TRUE("true");
 const std::string A2_V_FALSE("false");
+const std::string A2_V_DEFAULT("default");
 const std::string V_NONE("none");
 const std::string V_MEM("mem");
 const std::string V_ALL("all");
+
 /**
  * General preferences
  */
@@ -154,28 +156,40 @@ const std::string V_KQUEUE("kqueue");
 const std::string V_PORT("port");
 const std::string V_POLL("poll");
 const std::string V_SELECT("select");
+// value: true | false
+const std::string PREF_ENABLE_RPC("enable-rpc");
 // value: 1*digit
-const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
+const std::string PREF_RPC_LISTEN_PORT("rpc-listen-port");
+// value: string
+const std::string PREF_RPC_USER("rpc-user");
+// value: string
+const std::string PREF_RPC_PASSWD("rpc-passwd");
+// value: 1*digit
+const std::string PREF_RPC_MAX_REQUEST_SIZE("rpc-max-request-size");
+// value: true | false
+const std::string PREF_RPC_LISTEN_ALL("rpc-listen-all");
 // value: true | false
 const std::string PREF_ENABLE_XML_RPC("enable-xml-rpc");
-// value: true | false
-const std::string PREF_DRY_RUN("dry-run");
-// value: true | false
-const std::string PREF_REUSE_URI("reuse-uri");
+// value: 1*digit
+const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
 // value: string
 const std::string PREF_XML_RPC_USER("xml-rpc-user");
 // value: string
 const std::string PREF_XML_RPC_PASSWD("xml-rpc-passwd");
 // value: 1*digit
 const std::string PREF_XML_RPC_MAX_REQUEST_SIZE("xml-rpc-max-request-size");
+// value: true | false
+const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
+// value: true | false
+const std::string PREF_DRY_RUN("dry-run");
+// value: true | false
+const std::string PREF_REUSE_URI("reuse-uri");
 // value: string
 const std::string PREF_ON_DOWNLOAD_START("on-download-start");
 const std::string PREF_ON_DOWNLOAD_PAUSE("on-download-pause");
 const std::string PREF_ON_DOWNLOAD_STOP("on-download-stop");
 const std::string PREF_ON_DOWNLOAD_COMPLETE("on-download-complete");
 const std::string PREF_ON_DOWNLOAD_ERROR("on-download-error");
-// value: true | false
-const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
 // value: string
 const std::string PREF_INTERFACE("interface");
 // value: true | false
@@ -202,6 +216,18 @@ const std::string PREF_SELECT_LEAST_USED_HOST("select-least-used-host");
 const std::string PREF_ENABLE_ASYNC_DNS6("enable-async-dns6");
 // value: 1*digit
 const std::string PREF_MAX_DOWNLOAD_RESULT("max-download-result");
+// value: 1*digit
+const std::string PREF_RETRY_WAIT("retry-wait");
+// value: string
+const std::string PREF_ASYNC_DNS_SERVER("async-dns-server");
+// value: true | false
+const std::string PREF_SHOW_CONSOLE_READOUT("show-console-readout");
+// value: default | inorder
+const std::string PREF_STREAM_PIECE_SELECTOR("stream-piece-selector");
+// value: true | false
+const std::string PREF_TRUNCATE_CONSOLE_READOUT("truncate-console-readout");
+// value: true | false
+const std::string PREF_PAUSE("pause");
 
 /**
  * FTP related preferences
@@ -405,5 +431,6 @@ const std::string V_HTTPS("https");
 const std::string V_FTP("ftp");
 // values: true | false
 const std::string PREF_METALINK_ENABLE_UNIQUE_PROTOCOL("metalink-enable-unique-protocol");
+const std::string PREF_METALINK_BASE_URI("metalink-base-uri");
 
 } // namespace aria2
