@@ -57,10 +57,10 @@
 
 #ifdef HAVE_LIBGCRYPT
 # include <gcrypt.h>
-#elif HAVE_LIBSSL
+#elif HAVE_OPENSSL
 # include <openssl/rand.h>
 # include "SimpleRandomizer.h"
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 
 #include "File.h"
 #include "message.h"
@@ -84,7 +84,7 @@
 
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "MessageDigest.h"
-# include "MessageDigestHelper.h"
+# include "message_digest_helper.h"
 #endif // ENABLE_MESSAGE_DIGEST
 
 // For libc6 which doesn't define ULLONG_MAX properly because of broken limits.h
@@ -282,7 +282,7 @@ bool isUtf8(const std::string& str)
        firstChar == 0x09u || firstChar == 0x0au ||firstChar == 0x0du) {
       // UTF8-1 (without ctrl chars)
     } else if(in(firstChar, 0xc2u, 0xdfu)) {
-       // UTF8-2
+      // UTF8-2
       if(++s == eos || !isUtf8Tail(*s)) {
         return false;
       }
@@ -986,6 +986,22 @@ std::string getHomeDir()
   if(p) {
     return p;
   } else {
+#ifdef __MINGW32__
+    p = getenv("USERPROFILE");
+    if(p) {
+      return p;
+    } else {
+      p = getenv("HOMEDRIVE");
+      if(p) {
+        std::string homeDir = p;
+        p = getenv("HOMEPATH");
+        if(p) {
+          homeDir += p;
+          return homeDir;
+        }
+      }
+    }
+#endif
     return A2STR::NIL;
   }
 }
@@ -1269,7 +1285,7 @@ void generateRandomData(unsigned char* data, size_t length)
 {
 #ifdef HAVE_LIBGCRYPT
   gcry_randomize(data, length, GCRY_STRONG_RANDOM);
-#elif HAVE_LIBSSL
+#elif HAVE_OPENSSL
   if(RAND_bytes(data, length) != 1) {
     for(size_t i = 0; i < length; ++i) {
       data[i] = SimpleRandomizer::getInstance()->getRandomNumber(UINT8_MAX+1);
@@ -1278,7 +1294,7 @@ void generateRandomData(unsigned char* data, size_t length)
 #else
   std::ifstream i("/dev/urandom", std::ios::binary);
   i.read(reinterpret_cast<char*>(data), length);
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 }
 
 bool saveAs
@@ -1324,8 +1340,7 @@ void generateRandomKey(unsigned char* key)
 #ifdef ENABLE_MESSAGE_DIGEST
   unsigned char bytes[40];
   generateRandomData(bytes, sizeof(bytes));
-  MessageDigestHelper::digest
-    (key, 20, MessageDigest::sha1(), bytes, sizeof(bytes));
+  message_digest::digest(key, 20, MessageDigest::sha1(), bytes, sizeof(bytes));
 #else // !ENABLE_MESSAGE_DIGEST
   generateRandomData(key, 20);
 #endif // !ENABLE_MESSAGE_DIGEST
@@ -1444,6 +1459,11 @@ bool inSameCidrBlock(const std::string& ip1, const std::string& ip2, int bits)
 }
 
 void removeMetalinkContentTypes(const SharedHandle<RequestGroup>& group)
+{
+  removeMetalinkContentTypes(group.get());
+}
+
+void removeMetalinkContentTypes(RequestGroup* group)
 {
   for(std::vector<std::string>::const_iterator i =
 	DownloadHandlerConstants::getMetalinkContentTypes().begin(),
@@ -1457,18 +1477,18 @@ namespace {
 
 void executeHook
 (const std::string& command,
- gid_t gid,
+ a2_gid_t gid,
  size_t numFiles,
  const std::string& firstFilename)
 {
   const std::string gidStr = util::itos(gid);
   const std::string numFilesStr = util::uitos(numFiles);
+#ifndef __MINGW32__
   A2_LOG_INFO(fmt("Executing user command: %s %s %s %s",
                   command.c_str(),
                   gidStr.c_str(),
                   numFilesStr.c_str(),
                   firstFilename.c_str()));
-#ifndef __MINGW32__
   pid_t cpid = fork();
   if(cpid == -1) {
     A2_LOG_ERROR("fork() failed. Cannot execute user command.");
@@ -1491,11 +1511,30 @@ void executeHook
 
   memset(&pi, 0, sizeof (pi));
 
-  std::string cmdline = command;
+  bool batch = util::endsWith(util::toLower(command), ".bat");
+  std::string cmdline;
+  std::string cmdexe;
+  if(batch) {
+    const char* p = getenv("windir");
+    if(p) {
+      cmdexe = std::string(p);
+      cmdexe += "\\system32\\cmd.exe";
+    } else {
+      A2_LOG_INFO("Failed to get windir environment variable."
+                  " Executing batch file will fail.");
+      // TODO Might be useless.
+      cmdexe = "cmd.exe";
+    }
+    cmdline += "/C \"";
+  }
+  strappend(cmdline, "\"", command, "\"");
   strappend(cmdline, " ", gidStr, " ", numFilesStr, " \"", firstFilename, "\"");
-
+  if(batch) {
+    cmdline += "\"";
+  }
+  A2_LOG_INFO(fmt("Executing user command: %s", cmdline.c_str()));
   DWORD rc = CreateProcess(
-                           NULL,
+                           batch ? cmdexe.c_str() : NULL,
                            (LPSTR)cmdline.c_str(),
                            NULL,
                            NULL,
