@@ -1,8 +1,9 @@
+#include "HttpResponse.h"
+
 #include <iostream>
 
 #include <cppunit/extensions/HelperMacros.h>
 
-#include "HttpResponse.h"
 #include "prefs.h"
 #include "PiecedSegment.h"
 #include "Piece.h"
@@ -11,10 +12,14 @@
 #include "HttpRequest.h"
 #include "Exception.h"
 #include "A2STR.h"
-#include "Decoder.h"
 #include "DlRetryEx.h"
 #include "CookieStorage.h"
 #include "AuthConfigFactory.h"
+#include "AuthConfig.h"
+#include "StreamFilter.h"
+#include "MetalinkHttpEntry.h"
+#include "Option.h"
+#include "Checksum.h"
 
 namespace aria2 {
 
@@ -34,18 +39,23 @@ class HttpResponseTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testIsRedirect);
   CPPUNIT_TEST(testIsTransferEncodingSpecified);
   CPPUNIT_TEST(testGetTransferEncoding);
-  CPPUNIT_TEST(testGetTransferEncodingDecoder);
+  CPPUNIT_TEST(testGetTransferEncodingStreamFilter);
   CPPUNIT_TEST(testIsContentEncodingSpecified);
   CPPUNIT_TEST(testGetContentEncoding);
-  CPPUNIT_TEST(testGetContentEncodingDecoder);
+  CPPUNIT_TEST(testGetContentEncodingStreamFilter);
   CPPUNIT_TEST(testValidateResponse);
   CPPUNIT_TEST(testValidateResponse_good_range);
   CPPUNIT_TEST(testValidateResponse_bad_range);
   CPPUNIT_TEST(testValidateResponse_chunked);
+  CPPUNIT_TEST(testValidateResponse_withIfModifiedSince);
   CPPUNIT_TEST(testHasRetryAfter);
   CPPUNIT_TEST(testProcessRedirect);
   CPPUNIT_TEST(testRetrieveCookie);
   CPPUNIT_TEST(testSupportsPersistentConnection);
+  CPPUNIT_TEST(testGetMetalinKHttpEntries);
+#ifdef ENABLE_MESSAGE_DIGEST
+  CPPUNIT_TEST(testGetDigest);
+#endif // ENABLE_MESSAGE_DIGEST
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -65,18 +75,23 @@ public:
   void testIsRedirect();
   void testIsTransferEncodingSpecified();
   void testGetTransferEncoding();
-  void testGetTransferEncodingDecoder();
+  void testGetTransferEncodingStreamFilter();
   void testIsContentEncodingSpecified();
   void testGetContentEncoding();
-  void testGetContentEncodingDecoder();
+  void testGetContentEncodingStreamFilter();
   void testValidateResponse();
   void testValidateResponse_good_range();
   void testValidateResponse_bad_range();
   void testValidateResponse_chunked();
+  void testValidateResponse_withIfModifiedSince();
   void testHasRetryAfter();
   void testProcessRedirect();
   void testRetrieveCookie();
   void testSupportsPersistentConnection();
+  void testGetMetalinKHttpEntries();
+#ifdef ENABLE_MESSAGE_DIGEST
+  void testGetDigest();
+#endif // ENABLE_MESSAGE_DIGEST
 };
 
 
@@ -126,7 +141,7 @@ void HttpResponseTest::testGetContentType()
   httpResponse.setHttpHeader(httpHeader);
   // See paramter is ignored.
   CPPUNIT_ASSERT_EQUAL(std::string("application/metalink+xml"),
-		       httpResponse.getContentType());
+                       httpResponse.getContentType());
 }
 
 void HttpResponseTest::testDeterminFilename_without_ContentDisposition()
@@ -135,14 +150,14 @@ void HttpResponseTest::testDeterminFilename_without_ContentDisposition()
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
   SharedHandle<HttpRequest> httpRequest(new HttpRequest());
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
 
   httpResponse.setHttpHeader(httpHeader);
   httpResponse.setHttpRequest(httpRequest);
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-1.0.0.tar.bz2"),
-		       httpResponse.determinFilename());
+                       httpResponse.determinFilename());
 }
 
 void HttpResponseTest::testDeterminFilename_with_ContentDisposition_zero_length
@@ -153,14 +168,14 @@ void HttpResponseTest::testDeterminFilename_with_ContentDisposition_zero_length
   httpHeader->put("Content-Disposition", "attachment; filename=\"\"");
   SharedHandle<HttpRequest> httpRequest(new HttpRequest());
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
 
   httpResponse.setHttpHeader(httpHeader);
   httpResponse.setHttpRequest(httpRequest);
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-1.0.0.tar.bz2"),
-		       httpResponse.determinFilename());
+                       httpResponse.determinFilename());
 }
 
 void HttpResponseTest::testDeterminFilename_with_ContentDisposition()
@@ -168,17 +183,17 @@ void HttpResponseTest::testDeterminFilename_with_ContentDisposition()
   HttpResponse httpResponse;
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
   httpHeader->put("Content-Disposition",
-		  "attachment; filename=\"aria2-current.tar.bz2\"");
+                  "attachment; filename=\"aria2-current.tar.bz2\"");
   SharedHandle<HttpRequest> httpRequest(new HttpRequest());
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
 
   httpResponse.setHttpHeader(httpHeader);
   httpResponse.setHttpRequest(httpRequest);
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-current.tar.bz2"),
-		       httpResponse.determinFilename());
+                       httpResponse.determinFilename());
 }
 
 void HttpResponseTest::testGetRedirectURI_without_Location()
@@ -189,7 +204,7 @@ void HttpResponseTest::testGetRedirectURI_without_Location()
   httpResponse.setHttpHeader(httpHeader);
 
   CPPUNIT_ASSERT_EQUAL(std::string(""),
-		       httpResponse.getRedirectURI());  
+                       httpResponse.getRedirectURI());  
 }
 
 void HttpResponseTest::testGetRedirectURI_with_Location()
@@ -208,14 +223,14 @@ void HttpResponseTest::testIsRedirect()
 {
   HttpResponse httpResponse;
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
-  httpHeader->setResponseStatus("200");
+  httpHeader->setStatusCode(200);
   httpHeader->put("Location", "http://localhost/download/aria2-1.0.0.tar.bz2");
 
   httpResponse.setHttpHeader(httpHeader);
 
   CPPUNIT_ASSERT(!httpResponse.isRedirect());
 
-  httpHeader->setResponseStatus("304");
+  httpHeader->setStatusCode(301);
 
   CPPUNIT_ASSERT(httpResponse.isRedirect());  
 }
@@ -246,21 +261,21 @@ void HttpResponseTest::testGetTransferEncoding()
   httpHeader->put("Transfer-Encoding", "chunked");
 
   CPPUNIT_ASSERT_EQUAL(std::string("chunked"),
-		       httpResponse.getTransferEncoding());
+                       httpResponse.getTransferEncoding());
 }
 
-void HttpResponseTest::testGetTransferEncodingDecoder()
+void HttpResponseTest::testGetTransferEncodingStreamFilter()
 {
   HttpResponse httpResponse;
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
 
   httpResponse.setHttpHeader(httpHeader);
 
-  CPPUNIT_ASSERT(httpResponse.getTransferEncodingDecoder().isNull());  
+  CPPUNIT_ASSERT(!httpResponse.getTransferEncodingStreamFilter());
 
   httpHeader->put("Transfer-Encoding", "chunked");
 
-  CPPUNIT_ASSERT(!httpResponse.getTransferEncodingDecoder().isNull());
+  CPPUNIT_ASSERT(httpResponse.getTransferEncodingStreamFilter());
 }
 
 void HttpResponseTest::testIsContentEncodingSpecified()
@@ -291,37 +306,42 @@ void HttpResponseTest::testGetContentEncoding()
   CPPUNIT_ASSERT_EQUAL(std::string("gzip"), httpResponse.getContentEncoding());
 }
 
-void HttpResponseTest::testGetContentEncodingDecoder()
+void HttpResponseTest::testGetContentEncodingStreamFilter()
 {
   HttpResponse httpResponse;
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
   
   httpResponse.setHttpHeader(httpHeader);
 
-  CPPUNIT_ASSERT(httpResponse.getContentEncodingDecoder().isNull());
+  CPPUNIT_ASSERT(!httpResponse.getContentEncodingStreamFilter());
 
-#ifdef HAVE_LIBZ
+#ifdef HAVE_ZLIB
   httpHeader->put("Content-Encoding", "gzip");
   {
-    SharedHandle<Decoder> decoder = httpResponse.getContentEncodingDecoder();
-    CPPUNIT_ASSERT(!decoder.isNull());
-    CPPUNIT_ASSERT_EQUAL(std::string("GZipDecoder"), decoder->getName());
+    SharedHandle<StreamFilter> filter =
+      httpResponse.getContentEncodingStreamFilter();
+    CPPUNIT_ASSERT(filter);
+    CPPUNIT_ASSERT_EQUAL(std::string("GZipDecodingStreamFilter"),
+                         filter->getName());
   }
   httpHeader.reset(new HttpHeader());
   httpResponse.setHttpHeader(httpHeader);
   httpHeader->put("Content-Encoding", "deflate");
   {
-    SharedHandle<Decoder> decoder = httpResponse.getContentEncodingDecoder();
-    CPPUNIT_ASSERT(!decoder.isNull());
-    CPPUNIT_ASSERT_EQUAL(std::string("GZipDecoder"), decoder->getName());
+    SharedHandle<StreamFilter> filter =
+      httpResponse.getContentEncodingStreamFilter();
+    CPPUNIT_ASSERT(filter);
+    CPPUNIT_ASSERT_EQUAL(std::string("GZipDecodingStreamFilter"),
+                         filter->getName());
   }
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
   httpHeader.reset(new HttpHeader());
   httpResponse.setHttpHeader(httpHeader);
   httpHeader->put("Content-Encoding", "bzip2");
   {
-    SharedHandle<Decoder> decoder = httpResponse.getContentEncodingDecoder();
-    CPPUNIT_ASSERT(decoder.isNull());
+    SharedHandle<StreamFilter> filter =
+      httpResponse.getContentEncodingStreamFilter();
+    CPPUNIT_ASSERT(!filter);
   }
 }
 
@@ -331,7 +351,7 @@ void HttpResponseTest::testValidateResponse()
   SharedHandle<HttpHeader> httpHeader(new HttpHeader());
   httpResponse.setHttpHeader(httpHeader);
 
-  httpHeader->setResponseStatus("304");
+  httpHeader->setStatusCode(301);
 
   try {
     httpResponse.validateResponse();
@@ -357,12 +377,14 @@ void HttpResponseTest::testValidateResponse_good_range()
   SharedHandle<Piece> p(new Piece(1, 1024*1024));
   SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
   httpRequest->setSegment(segment);
+  SharedHandle<FileEntry> fileEntry(new FileEntry("file", 1024*1024*10, 0));
+  httpRequest->setFileEntry(fileEntry);
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
   httpResponse.setHttpRequest(httpRequest);
-  httpHeader->setResponseStatus("206");
-  httpHeader->put("Content-Range", "bytes 1048576-10485760/10485761");
+  httpHeader->setStatusCode(206);
+  httpHeader->put("Content-Range", "bytes 1048576-10485760/10485760");
   
   try {
     httpResponse.validateResponse();
@@ -382,11 +404,13 @@ void HttpResponseTest::testValidateResponse_bad_range()
   SharedHandle<Piece> p(new Piece(1, 1024*1024));
   SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
   httpRequest->setSegment(segment);
+  SharedHandle<FileEntry> fileEntry(new FileEntry("file", 1024*1024*10, 0));
+  httpRequest->setFileEntry(fileEntry);
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
   httpResponse.setHttpRequest(httpRequest);
-  httpHeader->setResponseStatus("206");
+  httpHeader->setStatusCode(206);
   httpHeader->put("Content-Range", "bytes 0-10485760/10485761");
 
   try {
@@ -406,11 +430,13 @@ void HttpResponseTest::testValidateResponse_chunked()
   SharedHandle<Piece> p(new Piece(1, 1024*1024));
   SharedHandle<Segment> segment(new PiecedSegment(1024*1024, p));
   httpRequest->setSegment(segment);
+  SharedHandle<FileEntry> fileEntry(new FileEntry("file", 1024*1024*10, 0));
+  httpRequest->setFileEntry(fileEntry);
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
   httpResponse.setHttpRequest(httpRequest);
-  httpHeader->setResponseStatus("206");
+  httpHeader->setStatusCode(206);
   httpHeader->put("Content-Range", "bytes 0-10485760/10485761");
   httpHeader->put("Transfer-Encoding", "chunked");
 
@@ -422,6 +448,23 @@ void HttpResponseTest::testValidateResponse_chunked()
   }
 }
 
+void HttpResponseTest::testValidateResponse_withIfModifiedSince()
+{
+  HttpResponse httpResponse;
+  SharedHandle<HttpHeader> httpHeader(new HttpHeader());
+  httpResponse.setHttpHeader(httpHeader);
+  httpHeader->setStatusCode(304);
+  SharedHandle<HttpRequest> httpRequest(new HttpRequest());
+  httpResponse.setHttpRequest(httpRequest);
+  try {
+    httpResponse.validateResponse();
+    CPPUNIT_FAIL("exception must be thrown.");
+  } catch(Exception& e) {
+  }
+  httpRequest->setIfModifiedSinceHeader("Fri, 16 Jul 2010 12:56:59 GMT");
+  httpResponse.validateResponse();
+}
+
 void HttpResponseTest::testHasRetryAfter()
 {
   HttpResponse httpResponse;
@@ -442,7 +485,7 @@ void HttpResponseTest::testProcessRedirect()
 
   SharedHandle<HttpRequest> httpRequest(new HttpRequest());
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://localhost/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
   httpResponse.setHttpRequest(httpRequest);
   
@@ -471,25 +514,26 @@ void HttpResponseTest::testRetrieveCookie()
 
   SharedHandle<HttpRequest> httpRequest(new HttpRequest());
   SharedHandle<Request> request(new Request());
-  request->setUrl("http://www.aria2.org/archives/aria2-1.0.0.tar.bz2");
+  request->setUri("http://www.aria2.org/archives/aria2-1.0.0.tar.bz2");
   httpRequest->setRequest(request);
   SharedHandle<CookieStorage> st(new CookieStorage());
   httpRequest->setCookieStorage(st);
   httpResponse.setHttpRequest(httpRequest);
 
   httpHeader->put("Set-Cookie", "k1=v1; expires=Sun, 10-Jun-2007 11:00:00 GMT;"
-		  "path=/; domain=.aria2.org;");
-  httpHeader->put("Set-Cookie", "k2=v2; expires=Sun, 01-Jan-09 00:00:00 GMT;"
-		  "path=/; domain=.aria2.org;");
+                  "path=/; domain=.aria2.org;");
+  httpHeader->put("Set-Cookie", "k2=v2; expires=Sun, 01-Jan-38 00:00:00 GMT;"
+                  "path=/; domain=.aria2.org;");
   httpHeader->put("Set-Cookie", "k3=v3;");
 
   httpResponse.retrieveCookie();
 
   CPPUNIT_ASSERT_EQUAL((size_t)2, st->size());
 
-  std::deque<Cookie>::const_iterator citer = st->begin();
-  CPPUNIT_ASSERT_EQUAL(std::string("k2=v2"), (*(st->begin())).toString());
-  CPPUNIT_ASSERT_EQUAL(std::string("k3=v3"), (*(st->begin()+1)).toString());
+  std::vector<Cookie> cookies;
+  st->dumpCookie(std::back_inserter(cookies));
+  CPPUNIT_ASSERT_EQUAL(std::string("k2=v2"), cookies[0].toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("k3=v3"), cookies[1].toString());
 }
 
 void HttpResponseTest::testSupportsPersistentConnection()
@@ -557,4 +601,85 @@ void HttpResponseTest::testSupportsPersistentConnection()
   httpHeader->clearField();
 }
 
+void HttpResponseTest::testGetMetalinKHttpEntries()
+{
+  HttpResponse httpResponse;
+  SharedHandle<HttpHeader> httpHeader(new HttpHeader());
+  httpResponse.setHttpHeader(httpHeader);
+  SharedHandle<Option> option(new Option());
+
+  httpHeader->put("Link", "<http://uri1/>; rel=duplicate; pri=1; pref; geo=JP");
+  httpHeader->put("Link", "<http://uri2/>; rel=duplicate");
+  httpHeader->put("Link", "<http://uri3/>;;;;;;;;rel=duplicate;;;;;pri=2;;;;;");
+  httpHeader->put("Link", "<http://uri4/>;rel=duplicate;=pri=1;pref");
+  httpHeader->put("Link", "<http://describedby>; rel=describedby");
+  httpHeader->put("Link", "<http://norel/>");
+  httpHeader->put("Link", "<baduri>; rel=duplicate; pri=-1;");
+  std::vector<MetalinkHttpEntry> result;
+  httpResponse.getMetalinKHttpEntries(result, option);
+  CPPUNIT_ASSERT_EQUAL((size_t)5, result.size());
+
+  MetalinkHttpEntry e = result[0];
+  CPPUNIT_ASSERT_EQUAL(std::string("http://uri1/"), e.uri);
+  CPPUNIT_ASSERT_EQUAL(1, e.pri);
+  CPPUNIT_ASSERT(e.pref);
+  CPPUNIT_ASSERT_EQUAL(std::string("jp"), e.geo);
+
+  e = result[1];
+  CPPUNIT_ASSERT_EQUAL(std::string("http://uri4/"), e.uri);
+  CPPUNIT_ASSERT_EQUAL(999999, e.pri);
+  CPPUNIT_ASSERT(e.pref);
+  CPPUNIT_ASSERT(e.geo.empty());
+
+  e = result[2];
+  CPPUNIT_ASSERT_EQUAL(std::string("http://uri3/"), e.uri);
+  CPPUNIT_ASSERT_EQUAL(2, e.pri);
+  CPPUNIT_ASSERT(!e.pref);
+  CPPUNIT_ASSERT(e.geo.empty());
+
+  e = result[3];
+  CPPUNIT_ASSERT_EQUAL(std::string("http://uri2/"), e.uri);
+  CPPUNIT_ASSERT_EQUAL(999999, e.pri);
+  CPPUNIT_ASSERT(!e.pref);
+  CPPUNIT_ASSERT(e.geo.empty());
+
+  e = result[4];
+  CPPUNIT_ASSERT_EQUAL(std::string("baduri"), e.uri);
+  CPPUNIT_ASSERT_EQUAL(999999, e.pri);
+  CPPUNIT_ASSERT(!e.pref);
+  CPPUNIT_ASSERT(e.geo.empty());
+}
+
+#ifdef ENABLE_MESSAGE_DIGEST
+void HttpResponseTest::testGetDigest()
+{
+  HttpResponse httpResponse;
+  SharedHandle<HttpHeader> httpHeader(new HttpHeader());
+  httpResponse.setHttpHeader(httpHeader);
+  SharedHandle<Option> option(new Option());
+  // Python binascii.hexlify(base64.b64decode(B64ED_HASH)) is handy to
+  // retrieve ascii hex hash string.
+  httpHeader->put("Digest", "SHA-1=82AD8itGL/oYQ5BTPFANiYnp9oE=");
+  httpHeader->put("Digest", "NOT_SUPPORTED");
+  httpHeader->put("Digest", "SHA-224=rQdowoLHQJTMVZ3rF7vmYOIzUXlu7F+FcMbPnA==");
+  httpHeader->put("Digest", "SHA-224=6Ik6LNZ1iPy6cbmlKO4NHfvxzaiurmHilMyhGA==");
+  httpHeader->put("Digest",
+                  "SHA-256=+D8nGudz3G/kpkVKQeDrI3xD57v0UeQmzGCZOk03nsU=,"
+                  "MD5=LJDK2+9ClF8Nz/K5WZd/+A==");
+  std::vector<Checksum> result;
+  httpResponse.getDigest(result);
+  CPPUNIT_ASSERT_EQUAL((size_t)3, result.size());
+
+  Checksum c = result[0];
+  CPPUNIT_ASSERT_EQUAL(std::string("sha-256"), c.getHashType());
+  CPPUNIT_ASSERT_EQUAL(std::string("f83f271ae773dc6fe4a6454a41e0eb237c43e7bbf451e426cc60993a4d379ec5"),
+                       c.getDigest());
+
+  c = result[1];
+  CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), c.getHashType());
+  CPPUNIT_ASSERT_EQUAL(std::string("f36003f22b462ffa184390533c500d8989e9f681"),
+                       c.getDigest());
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
 } // namespace aria2
