@@ -8,7 +8,6 @@
 #include "DiskAdaptor.h"
 #include "FileEntry.h"
 #include "PieceSelector.h"
-#include "messageDigest.h"
 
 namespace aria2 {
 
@@ -39,12 +38,12 @@ const char* IteratableChunkChecksumValidatorTest::csArray[] = { "29b0e7878271645
 void IteratableChunkChecksumValidatorTest::testValidate() {
   Option option;
   SharedHandle<DownloadContext> dctx
-    (new DownloadContext(100, 250, "chunkChecksumTestFile250.txt"));
-  dctx->setPieceHashes(&csArray[0], &csArray[3]);
-  dctx->setPieceHashAlgo(MessageDigestContext::SHA1);
+    (new DownloadContext(100, 250, A2_TEST_DIR"/chunkChecksumTestFile250.txt"));
+  dctx->setPieceHashes("sha-1", &csArray[0], &csArray[3]);
   SharedHandle<DefaultPieceStorage> ps
     (new DefaultPieceStorage(dctx, &option));
   ps->initStorage();
+  ps->getDiskAdaptor()->enableReadOnly();
   ps->getDiskAdaptor()->openFile();
 
   IteratableChunkChecksumValidator validator(dctx, ps);
@@ -61,7 +60,7 @@ void IteratableChunkChecksumValidatorTest::testValidate() {
   // make the test fail
   std::deque<std::string> badHashes(&csArray[0], &csArray[3]);
   badHashes[1] = "ffffffffffffffffffffffffffffffffffffffff";
-  dctx->setPieceHashes(badHashes.begin(), badHashes.end());
+  dctx->setPieceHashes("sha-1", badHashes.begin(), badHashes.end());
 
   validator.init();
 
@@ -76,14 +75,14 @@ void IteratableChunkChecksumValidatorTest::testValidate() {
 void IteratableChunkChecksumValidatorTest::testValidate_readError() {
   Option option;
   SharedHandle<DownloadContext> dctx
-    (new DownloadContext(100, 500, "chunkChecksumTestFile250.txt"));
+    (new DownloadContext(100, 500, A2_TEST_DIR"/chunkChecksumTestFile250.txt"));
   std::deque<std::string> hashes(&csArray[0], &csArray[3]);
   hashes.push_back("ffffffffffffffffffffffffffffffffffffffff");
   hashes.push_back("ffffffffffffffffffffffffffffffffffffffff");
-  dctx->setPieceHashes(hashes.begin(), hashes.end());
-  dctx->setPieceHashAlgo(MessageDigestContext::SHA1);
+  dctx->setPieceHashes("sha-1", hashes.begin(), hashes.end());
   SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, &option));
   ps->initStorage();
+  ps->getDiskAdaptor()->enableReadOnly();
   ps->getDiskAdaptor()->openFile();
 
   IteratableChunkChecksumValidator validator(dctx, ps);
