@@ -1,5 +1,5 @@
-#ifndef _D_MOCK_PIECE_STORAGE_H_
-#define _D_MOCK_PIECE_STORAGE_H_
+#ifndef D_MOCK_PIECE_STORAGE_H
+#define D_MOCK_PIECE_STORAGE_H
 
 #include "PieceStorage.h"
 
@@ -44,22 +44,47 @@ public:
     return false;
   }
 
-  virtual SharedHandle<Piece> getMissingPiece(const SharedHandle<Peer>& peer) {
-    return SharedHandle<Piece>(new Piece());
-  }
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid)
+  {}
+
+  virtual void getMissingPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid)
+  {}
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   cuid_t cuid)
+  {}
+
+  virtual void getMissingFastPiece
+  (std::vector<SharedHandle<Piece> >& pieces,
+   size_t minMissingBlocks,
+   const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid)
+  {}
 
   virtual SharedHandle<Piece> getMissingPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+  (const SharedHandle<Peer>& peer,
+   cuid_t cuid)
   {
     return SharedHandle<Piece>(new Piece());
   }
 
-  virtual SharedHandle<Piece> getMissingFastPiece(const SharedHandle<Peer>& peer) {
-    return SharedHandle<Piece>(new Piece());
-  }
-
-  virtual SharedHandle<Piece> getMissingFastPiece
-  (const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+  virtual SharedHandle<Piece> getMissingPiece
+  (const SharedHandle<Peer>& peer,
+   const std::vector<size_t>& excludedIndexes,
+   cuid_t cuid)
   {
     return SharedHandle<Piece>(new Piece());
   }
@@ -71,13 +96,16 @@ public:
     return false;
   }
 
-  virtual SharedHandle<Piece> getSparseMissingUnusedPiece
-  (size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length)
+  virtual SharedHandle<Piece> getMissingPiece
+  (size_t minSplitSize,
+   const unsigned char* ignoreBitfield,
+   size_t length,
+   cuid_t cuid)
   {
     return SharedHandle<Piece>(new Piece());
   }
 
-  virtual SharedHandle<Piece> getMissingPiece(size_t index)
+  virtual SharedHandle<Piece> getMissingPiece(size_t index, cuid_t cuid)
   {
     return SharedHandle<Piece>(new Piece());
   }
@@ -97,7 +125,7 @@ public:
 
   virtual void completePiece(const SharedHandle<Piece>& piece) {}
 
-  virtual void cancelPiece(const SharedHandle<Piece>& piece) {}
+  virtual void cancelPiece(const SharedHandle<Piece>& piece, cuid_t cuid) {}
 
   virtual bool hasPiece(size_t index) {
     return false;
@@ -190,8 +218,8 @@ public:
 
   virtual void setEndGamePieceNum(size_t num) {}
 
-  void setEndGame(bool flag) {
-    this->endGame = flag;
+  virtual void enterEndGame() {
+    this->endGame = true;
   }
 
   virtual SharedHandle<DiskAdaptor> getDiskAdaptor() {
@@ -256,4 +284,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_MOCK_PIECE_STORAGE_H_
+#endif // D_MOCK_PIECE_STORAGE_H
