@@ -2,10 +2,13 @@
 
 #include <string>
 
+#include "SharedHandle.h"
 #include "Cookie.h"
 
 namespace aria2 {
 
+class MessageDigest;
+
 void createFile(const std::string& filename, size_t length);
 
 std::string readFile(const std::string& path);
@@ -39,4 +42,10 @@ Cookie createCookie
  const std::string& path,
  bool secure);
 
+#ifdef ENABLE_MESSAGE_DIGEST
+// Returns hex digest of contents of file denoted by filename.
+std::string fileHexDigest
+(const SharedHandle<MessageDigest>& ctx, const std::string& filename);
+#endif // ENABLE_MESSAGE_DIGEST
+
 } // namespace aria2
