@@ -19,9 +19,9 @@ namespace aria2 {
 class UtilTest:public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(UtilTest);
-  CPPUNIT_TEST(testTrim);
+  CPPUNIT_TEST(testStrip);
+  CPPUNIT_TEST(testDivide);
   CPPUNIT_TEST(testSplit);
-  CPPUNIT_TEST(testSplit_many);
   CPPUNIT_TEST(testEndsWith);
   CPPUNIT_TEST(testReplace);
   CPPUNIT_TEST(testStartsWith);
@@ -30,7 +30,9 @@ class UtilTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testRandomAlpha);
   CPPUNIT_TEST(testToUpper);
   CPPUNIT_TEST(testToLower);
-  CPPUNIT_TEST(testUrldecode);
+  CPPUNIT_TEST(testUppercase);
+  CPPUNIT_TEST(testLowercase);
+  CPPUNIT_TEST(testPercentDecode);
   CPPUNIT_TEST(testGetRealSize);
   CPPUNIT_TEST(testAbbrevSize);
   CPPUNIT_TEST(testToStream);
@@ -46,11 +48,14 @@ class UtilTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testParseUInt);
   CPPUNIT_TEST(testParseLLInt);
   CPPUNIT_TEST(testParseULLInt);
+  CPPUNIT_TEST(testParseIntNoThrow);
+  CPPUNIT_TEST(testParseUIntNoThrow);
+  CPPUNIT_TEST(testParseLLIntNoThrow);
   CPPUNIT_TEST(testToString_binaryStream);
   CPPUNIT_TEST(testItos);
   CPPUNIT_TEST(testUitos);
   CPPUNIT_TEST(testNtoh64);
-  CPPUNIT_TEST(testUrlencode);
+  CPPUNIT_TEST(testPercentEncode);
   CPPUNIT_TEST(testHtmlEscape);
   CPPUNIT_TEST(testJoinPath);
   CPPUNIT_TEST(testParseIndexPath);
@@ -61,6 +66,12 @@ class UtilTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testApplyDir);
   CPPUNIT_TEST(testFixTaintedBasename);
   CPPUNIT_TEST(testIsNumericHost);
+  CPPUNIT_TEST(testDetectDirTraversal);
+  CPPUNIT_TEST(testEscapePath);
+  CPPUNIT_TEST(testGetCidrPrefix);
+  CPPUNIT_TEST(testInSameCidrBlock);
+  CPPUNIT_TEST(testIsUtf8String);
+  CPPUNIT_TEST(testNextParam);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -68,9 +79,9 @@ public:
   void setUp() {
   }
 
-  void testTrim();
+  void testStrip();
+  void testDivide();
   void testSplit();
-  void testSplit_many();
   void testEndsWith();
   void testReplace();
   void testStartsWith();
@@ -79,7 +90,9 @@ public:
   void testRandomAlpha();
   void testToUpper();
   void testToLower();
-  void testUrldecode();
+  void testUppercase();
+  void testLowercase();
+  void testPercentDecode();
   void testGetRealSize();
   void testAbbrevSize();
   void testToStream();
@@ -95,11 +108,14 @@ public:
   void testParseUInt();
   void testParseLLInt();
   void testParseULLInt();
+  void testParseIntNoThrow();
+  void testParseUIntNoThrow();
+  void testParseLLIntNoThrow();
   void testToString_binaryStream();
   void testItos();
   void testUitos();
   void testNtoh64();
-  void testUrlencode();
+  void testPercentEncode();
   void testHtmlEscape();
   void testJoinPath();
   void testParseIndexPath();
@@ -110,85 +126,146 @@ public:
   void testApplyDir();
   void testFixTaintedBasename();
   void testIsNumericHost();
+  void testDetectDirTraversal();
+  void testEscapePath();
+  void testGetCidrPrefix();
+  void testInSameCidrBlock();
+  void testIsUtf8String();
+  void testNextParam();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION( UtilTest );
 
-void UtilTest::testTrim() {
+void UtilTest::testStrip()
+{
   std::string str1 = "aria2";
-  CPPUNIT_ASSERT_EQUAL(str1, util::trim("aria2"));
-  CPPUNIT_ASSERT_EQUAL(str1, util::trim(" aria2"));
-  CPPUNIT_ASSERT_EQUAL(str1, util::trim(" aria2 "));
-  CPPUNIT_ASSERT_EQUAL(str1, util::trim("  aria2  "));
+  CPPUNIT_ASSERT_EQUAL(str1, util::strip("aria2"));
+  CPPUNIT_ASSERT_EQUAL(str1, util::strip(" aria2"));
+  CPPUNIT_ASSERT_EQUAL(str1, util::strip("aria2 "));
+  CPPUNIT_ASSERT_EQUAL(str1, util::strip(" aria2 "));
+  CPPUNIT_ASSERT_EQUAL(str1, util::strip("  aria2  "));
   std::string str2 = "aria2 debut";
-  CPPUNIT_ASSERT_EQUAL(str2, util::trim("aria2 debut"));
-  CPPUNIT_ASSERT_EQUAL(str2, util::trim(" aria2 debut "));
+  CPPUNIT_ASSERT_EQUAL(str2, util::strip("aria2 debut"));
+  CPPUNIT_ASSERT_EQUAL(str2, util::strip(" aria2 debut "));
   std::string str3 = "";
-  CPPUNIT_ASSERT_EQUAL(str3, util::trim(""));
-  CPPUNIT_ASSERT_EQUAL(str3, util::trim(" "));
-  CPPUNIT_ASSERT_EQUAL(str3, util::trim("  "));
+  CPPUNIT_ASSERT_EQUAL(str3, util::strip(""));
+  CPPUNIT_ASSERT_EQUAL(str3, util::strip(" "));
+  CPPUNIT_ASSERT_EQUAL(str3, util::strip("  "));
   std::string str4 = "A";
-  CPPUNIT_ASSERT_EQUAL(str4, util::trim("A"));
-  CPPUNIT_ASSERT_EQUAL(str4, util::trim(" A "));
-  CPPUNIT_ASSERT_EQUAL(str4, util::trim("  A  "));
+  CPPUNIT_ASSERT_EQUAL(str4, util::strip("A"));
+  CPPUNIT_ASSERT_EQUAL(str4, util::strip(" A "));
+  CPPUNIT_ASSERT_EQUAL(str4, util::strip("  A  "));
+}
+
+void UtilTest::testDivide() {
+  std::pair<std::string, std::string> p1;
+  util::divide(p1, "name=value", '=');
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
+  util::divide(p1, " name = value ", '=');
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
+  util::divide(p1, "=value", '=');
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
+  util::divide(p1, "name=", '=');
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
+  util::divide(p1, "name", '=');
+  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
 }
 
 void UtilTest::testSplit() {
-  std::pair<std::string, std::string> p1;
-  util::split(p1, "name=value", '=');
-  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
-  util::split(p1, " name = value ", '=');
-  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
-  util::split(p1, "=value", '=');
-  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.first);
-  CPPUNIT_ASSERT_EQUAL(std::string("value"), p1.second);
-  util::split(p1, "name=", '=');
-  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
-  util::split(p1, "name", '=');
-  CPPUNIT_ASSERT_EQUAL(std::string("name"), p1.first);
-  CPPUNIT_ASSERT_EQUAL(std::string(""), p1.second);
-}
+  std::vector<std::string> v;
+  util::split("k1; k2;; k3", std::back_inserter(v), ";", true);
+  CPPUNIT_ASSERT_EQUAL((size_t)3, v.size());
+  std::vector<std::string>::iterator itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("k2"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("k3"), *itr++);
 
-void UtilTest::testSplit_many() {
-  std::vector<std::string> v1;
-  util::split("name1=value1; name2=value2; name3=value3",std::back_inserter(v1),
-              ";", true);
-  CPPUNIT_ASSERT_EQUAL((size_t)3, v1.size());
-  std::vector<std::string>::iterator itr = v1.begin();
-  CPPUNIT_ASSERT_EQUAL(std::string("name1=value1"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(std::string("name2=value2"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(std::string("name3=value3"), *itr++);
+  v.clear();
 
-  v1.clear();
+  util::split("k1; k2; k3",
+              std::back_inserter(v), ";");
+  CPPUNIT_ASSERT_EQUAL((size_t)3, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(" k2"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(" k3"), *itr++);
 
-  util::split("name1=value1; name2=value2; name3=value3",std::back_inserter(v1),
-              ";", false);
-  CPPUNIT_ASSERT_EQUAL((size_t)3, v1.size());
-  itr = v1.begin();
-  CPPUNIT_ASSERT_EQUAL(std::string("name1=value1"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(std::string(" name2=value2"), *itr++);
-  CPPUNIT_ASSERT_EQUAL(std::string(" name3=value3"), *itr++);
+  v.clear();
 
-  v1.clear();
+  util::split("k=v", std::back_inserter(v), ";", false, true);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k=v"), *itr++);
 
-  util::split("k=v", std::back_inserter(v1), ";", false, true);
-  CPPUNIT_ASSERT_EQUAL((size_t)1, v1.size());
-  CPPUNIT_ASSERT_EQUAL(std::string("k=v"), v1[0]);
+  v.clear();
 
-  v1.clear();
+  util::split(";;k1;;k2;", std::back_inserter(v), ";", false, true);
+  CPPUNIT_ASSERT_EQUAL((size_t)6, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("k1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("k2"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
 
-  util::split(" ", std::back_inserter(v1), ";", true, true);
-  CPPUNIT_ASSERT_EQUAL((size_t)1, v1.size());
-  CPPUNIT_ASSERT_EQUAL(std::string(""), v1[0]);
+  v.clear();
 
-  v1.clear();
+  util::split(";;k1;;k2;", std::back_inserter(v), ";");
+  CPPUNIT_ASSERT_EQUAL((size_t)2, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k1"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string("k2"), *itr++);
 
-  util::split(" ", std::back_inserter(v1), ";", true);
-  CPPUNIT_ASSERT_EQUAL((size_t)0, v1.size());
+  v.clear();
+
+  util::split("k; ", std::back_inserter(v), ";");
+  CPPUNIT_ASSERT_EQUAL((size_t)2, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string("k"), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(" "), *itr++);
+
+  v.clear();
+
+  util::split(" ", std::back_inserter(v), ";", true, true);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, v.size());
+  CPPUNIT_ASSERT_EQUAL(std::string(""), v[0]);
+
+  v.clear();
+
+  util::split(" ", std::back_inserter(v), ";", true);
+  CPPUNIT_ASSERT_EQUAL((size_t)0, v.size());
+
+  v.clear();
+
+  util::split(" ", std::back_inserter(v), ";");
+  CPPUNIT_ASSERT_EQUAL((size_t)1, v.size());
+  CPPUNIT_ASSERT_EQUAL(std::string(" "), v[0]);
+
+  v.clear();
+
+  util::split(";", std::back_inserter(v), ";");
+  CPPUNIT_ASSERT_EQUAL((size_t)0, v.size());
+
+  v.clear();
+
+  util::split(";", std::back_inserter(v), ";", false, true);
+  CPPUNIT_ASSERT_EQUAL((size_t)2, v.size());
+  itr = v.begin();
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
+  CPPUNIT_ASSERT_EQUAL(std::string(""), *itr++);
+
+  v.clear();
+
+  util::split("", std::back_inserter(v), ";", false, true);
+  CPPUNIT_ASSERT_EQUAL((size_t)1, v.size());
+  CPPUNIT_ASSERT_EQUAL(std::string(""), v[0]);
 }
 
 void UtilTest::testEndsWith() {
@@ -293,7 +370,7 @@ void UtilTest::testGetContentDispositionFilename() {
   CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"), util::getContentDispositionFilename(h8));
 
   std::string h9 = "attachment; filename=\"aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT\"";
-  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2"),
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2.tar.bz2; creation-date=20 Jun 2007 00:00:00 GMT"),
                        util::getContentDispositionFilename(h9));
 
   std::string h10 = "attachment; filename=";
@@ -303,16 +380,17 @@ void UtilTest::testGetContentDispositionFilename() {
   CPPUNIT_ASSERT_EQUAL(std::string(""), util::getContentDispositionFilename(h11));
 
   std::string filenameWithDir = "attachment; filename=dir/file";
-  CPPUNIT_ASSERT_EQUAL(std::string("file"),
+  CPPUNIT_ASSERT_EQUAL(std::string(""),
                        util::getContentDispositionFilename(filenameWithDir));
 
-  std::string parentDir = "attachment; filename=..";
-  CPPUNIT_ASSERT_EQUAL(std::string(),
-                       util::getContentDispositionFilename(parentDir));
+  std::string semicolonInside = "attachment; filename=\"foo;bar\"";
+  CPPUNIT_ASSERT_EQUAL(std::string("foo;bar"),
+                       util::getContentDispositionFilename(semicolonInside));
+
+  CPPUNIT_ASSERT_EQUAL
+    (std::string(""),
+     util::getContentDispositionFilename("filename=\"%2E%2E%2Ffoo.html\""));
 
-  std::string currentDir = "attachment; filename=.";
-  CPPUNIT_ASSERT_EQUAL(std::string(),
-                       util::getContentDispositionFilename(currentDir));
   // RFC2231 Section4
   std::string extparam2 = "filename*=''aria2";
   CPPUNIT_ASSERT_EQUAL(std::string("aria2"),
@@ -341,6 +419,15 @@ void UtilTest::testGetContentDispositionFilename() {
   extparam9ans += 0xa3;
   CPPUNIT_ASSERT_EQUAL(extparam9ans,
                        util::getContentDispositionFilename(extparam9));
+  CPPUNIT_ASSERT_EQUAL
+    (std::string(""),
+     util::getContentDispositionFilename("filename*=UTF-8''foo%2F.html"));
+  CPPUNIT_ASSERT_EQUAL
+    (std::string("foo.html"),
+     util::getContentDispositionFilename("filename*=UTF-8'';filename=\"foo.html\""));
+  CPPUNIT_ASSERT_EQUAL
+    (std::string(""),
+     util::getContentDispositionFilename("filename*=UTF-8''%2E%2E%2Ffoo.html"));
 
   // Tests from http://greenbytes.de/tech/tc2231/
   // attwithasciifnescapedchar
@@ -412,25 +499,39 @@ void UtilTest::testToLower() {
   CPPUNIT_ASSERT_EQUAL(upp, util::toLower(src));
 }
 
-void UtilTest::testUrldecode() {
+void UtilTest::testUppercase() {
+  std::string src = "608cabc0f2fa18c260cafd974516865c772363d5";
+  std::string ans = "608CABC0F2FA18C260CAFD974516865C772363D5";
+  util::uppercase(src);
+  CPPUNIT_ASSERT_EQUAL(ans, src);
+}
+
+void UtilTest::testLowercase() {
+  std::string src = "608CABC0F2FA18C260CAFD974516865C772363D5";
+  std::string ans = "608cabc0f2fa18c260cafd974516865c772363d5";
+  util::lowercase(src);
+  CPPUNIT_ASSERT_EQUAL(ans, src);
+}
+
+void UtilTest::testPercentDecode() {
   std::string src = "http://aria2.sourceforge.net/aria2%200.7.0%20docs.html";
   CPPUNIT_ASSERT_EQUAL(std::string("http://aria2.sourceforge.net/aria2 0.7.0 docs.html"),
-                       util::urldecode(src));
+                       util::percentDecode(src));
 
   std::string src2 = "aria2+aria2";
-  CPPUNIT_ASSERT_EQUAL(std::string("aria2+aria2"), util::urldecode(src2));
+  CPPUNIT_ASSERT_EQUAL(std::string("aria2+aria2"), util::percentDecode(src2));
 
   std::string src3 = "%5t%20";
-  CPPUNIT_ASSERT_EQUAL(std::string("%5t "), util::urldecode(src3));
+  CPPUNIT_ASSERT_EQUAL(std::string("%5t "), util::percentDecode(src3));
 
   std::string src4 = "%";
-  CPPUNIT_ASSERT_EQUAL(std::string("%"), util::urldecode(src4));
+  CPPUNIT_ASSERT_EQUAL(std::string("%"), util::percentDecode(src4));
   
   std::string src5 = "%3";
-  CPPUNIT_ASSERT_EQUAL(std::string("%3"), util::urldecode(src5));
+  CPPUNIT_ASSERT_EQUAL(std::string("%3"), util::percentDecode(src5));
 
   std::string src6 = "%2f";
-  CPPUNIT_ASSERT_EQUAL(std::string("/"), util::urldecode(src6));
+  CPPUNIT_ASSERT_EQUAL(std::string("/"), util::percentDecode(src6));
 }
 
 void UtilTest::testGetRealSize()
@@ -543,7 +644,7 @@ void UtilTest::testAlphaToNum()
 
 void UtilTest::testMkdirs()
 {
-  std::string dir = "/tmp/aria2-UtilTest-testMkdirs";
+  std::string dir = "./aria2-UtilTest-testMkdirs";
   File d(dir);
   if(d.exists()) {
     CPPUNIT_ASSERT(d.remove());
@@ -552,7 +653,7 @@ void UtilTest::testMkdirs()
   util::mkdirs(dir);
   CPPUNIT_ASSERT(d.isDir());
 
-  std::string file = "./UtilTest.cc";
+  std::string file = A2_TEST_DIR"/UtilTest.cc";
   File f(file);
   CPPUNIT_ASSERT(f.isFile());
   try {
@@ -731,6 +832,42 @@ void UtilTest::testParseULLInt()
   }
 }
 
+void UtilTest::testParseIntNoThrow()
+{
+  int32_t n;
+  CPPUNIT_ASSERT(util::parseIntNoThrow(n, " -1 "));
+  CPPUNIT_ASSERT_EQUAL((int32_t)-1, n);
+
+  CPPUNIT_ASSERT(util::parseIntNoThrow(n, "2147483647"));
+  CPPUNIT_ASSERT_EQUAL((int32_t)2147483647, n);
+
+  CPPUNIT_ASSERT(!util::parseIntNoThrow(n, "2147483648"));
+  CPPUNIT_ASSERT(!util::parseIntNoThrow(n, "-2147483649"));
+
+  CPPUNIT_ASSERT(!util::parseIntNoThrow(n, "12x"));
+  CPPUNIT_ASSERT(!util::parseIntNoThrow(n, ""));
+}
+
+void UtilTest::testParseUIntNoThrow()
+{
+  uint32_t n;
+  CPPUNIT_ASSERT(util::parseUIntNoThrow(n, " 4294967295 "));
+  CPPUNIT_ASSERT_EQUAL((uint32_t)UINT32_MAX, n);
+  CPPUNIT_ASSERT(!util::parseUIntNoThrow(n, "4294967296"));
+  CPPUNIT_ASSERT(!util::parseUIntNoThrow(n, "-1"));
+}
+
+void UtilTest::testParseLLIntNoThrow()
+{
+  int64_t n;
+  CPPUNIT_ASSERT(util::parseLLIntNoThrow(n, " 9223372036854775807 "));
+  CPPUNIT_ASSERT_EQUAL((int64_t)INT64_MAX, n);
+  CPPUNIT_ASSERT(!util::parseLLIntNoThrow(n, "9223372036854775808"));
+  CPPUNIT_ASSERT(util::parseLLIntNoThrow(n, "-9223372036854775808"));
+  CPPUNIT_ASSERT_EQUAL((int64_t)INT64_MIN, n);
+  CPPUNIT_ASSERT(!util::parseLLIntNoThrow(n, "-9223372036854775809"));
+}
+
 void UtilTest::testToString_binaryStream()
 {
   SharedHandle<DiskWriter> dw(new ByteArrayDiskWriter());
@@ -771,8 +908,14 @@ void UtilTest::testItos()
   }
   {
     int64_t i = INT64_MAX;
-    CPPUNIT_ASSERT_EQUAL(std::string("9,223,372,036,854,775,807"), util::itos(i, true));
-  }  
+    CPPUNIT_ASSERT_EQUAL(std::string("9,223,372,036,854,775,807"),
+                         util::itos(i, true));
+  }
+  {
+    int64_t i = INT64_MIN;
+    CPPUNIT_ASSERT_EQUAL(std::string("-9,223,372,036,854,775,808"),
+                         util::itos(i, true));
+  }
 }
 
 void UtilTest::testUitos()
@@ -800,20 +943,20 @@ void UtilTest::testNtoh64()
 #endif // !WORDS_BIGENDIAN
 }
 
-void UtilTest::testUrlencode()
+void UtilTest::testPercentEncode()
 {
   CPPUNIT_ASSERT_EQUAL
     (std::string("%3A%2F%3F%23%5B%5D%40%21%25%26%27%28%29%2A%2B%2C%3B%3D"),
-     util::urlencode(":/?#[]@!%&'()*+,;="));
+     util::percentEncode(":/?#[]@!%&'()*+,;="));
 
   std::string unreserved =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "abcdefghijklmnopqrstuvwxyz"
     "0123456789"
     "-._~";
-  CPPUNIT_ASSERT_EQUAL(unreserved, util::urlencode(unreserved));
+  CPPUNIT_ASSERT_EQUAL(unreserved, util::percentEncode(unreserved));
 
-  CPPUNIT_ASSERT_EQUAL(std::string("1%5EA%20"), util::urlencode("1^A "));
+  CPPUNIT_ASSERT_EQUAL(std::string("1%5EA%20"), util::percentEncode("1^A "));
 }
 
 void UtilTest::testHtmlEscape()
@@ -827,33 +970,28 @@ void UtilTest::testJoinPath()
   const std::string dir1dir2file[] = { "dir1", "dir2", "file" };
   CPPUNIT_ASSERT_EQUAL
     (std::string("dir1/dir2/file"),
-     util::joinPath(&dir1dir2file[0],
-                    &dir1dir2file[arrayLength(dir1dir2file)]));
+     util::joinPath(vbegin(dir1dir2file), vend(dir1dir2file)));
 
   const std::string dirparentfile[] = { "dir", "..", "file" };
   CPPUNIT_ASSERT_EQUAL
     (std::string("file"),
-     util::joinPath(&dirparentfile[0],
-                    &dirparentfile[arrayLength(dirparentfile)]));
+     util::joinPath(vbegin(dirparentfile), vend(dirparentfile)));
 
   const std::string dirparentparentfile[] = { "dir", "..", "..", "file" };
   CPPUNIT_ASSERT_EQUAL
     (std::string("file"),
-     util::joinPath(&dirparentparentfile[0],
-                    &dirparentparentfile[arrayLength(dirparentparentfile)]));
+     util::joinPath(vbegin(dirparentparentfile), vend(dirparentparentfile)));
 
   const std::string dirdotfile[] = { "dir", ".", "file" };
   CPPUNIT_ASSERT_EQUAL(std::string("dir/file"),
-                       util::joinPath(&dirdotfile[0],
-                                      &dirdotfile[arrayLength(dirdotfile)]));
+                       util::joinPath(vbegin(dirdotfile), vend(dirdotfile)));
 
   const std::string empty[] = {};
   CPPUNIT_ASSERT_EQUAL(std::string(""), util::joinPath(&empty[0], &empty[0]));
 
   const std::string parentdot[] = { "..", "." };
   CPPUNIT_ASSERT_EQUAL(std::string(""),
-                       util::joinPath(&parentdot[0],
-                                      &parentdot[arrayLength(parentdot)]));
+                       util::joinPath(vbegin(parentdot), vend(parentdot)));
 }
 
 void UtilTest::testParseIndexPath()
@@ -1003,9 +1141,12 @@ void UtilTest::testApplyDir()
 
 void UtilTest::testFixTaintedBasename()
 {
-  CPPUNIT_ASSERT_EQUAL(std::string("a_b"), util::fixTaintedBasename("a/b"));
-  CPPUNIT_ASSERT_EQUAL(std::string("a_b"), util::fixTaintedBasename("a\\b"));
-  CPPUNIT_ASSERT_EQUAL(std::string("a__b"), util::fixTaintedBasename("a\\/b"));
+  CPPUNIT_ASSERT_EQUAL(std::string("a%2Fb"), util::fixTaintedBasename("a/b"));
+#ifdef __MINGW32__
+  CPPUNIT_ASSERT_EQUAL(std::string("a%5Cb"), util::fixTaintedBasename("a\\b"));
+#else // !__MINGW32__
+  CPPUNIT_ASSERT_EQUAL(std::string("a\\b"), util::fixTaintedBasename("a\\b"));
+#endif // !__MINGW32__
 }
 
 void UtilTest::testIsNumericHost()
@@ -1015,4 +1156,135 @@ void UtilTest::testIsNumericHost()
   CPPUNIT_ASSERT(util::isNumericHost("::1"));
 }
 
+void UtilTest::testDetectDirTraversal()
+{
+  CPPUNIT_ASSERT(util::detectDirTraversal("/foo"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("./foo"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("../foo"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("foo/../bar"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("foo/./bar"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("foo/."));
+  CPPUNIT_ASSERT(util::detectDirTraversal("foo/.."));
+  CPPUNIT_ASSERT(util::detectDirTraversal("."));
+  CPPUNIT_ASSERT(util::detectDirTraversal(".."));
+  CPPUNIT_ASSERT(util::detectDirTraversal("/"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("foo/"));
+  CPPUNIT_ASSERT(util::detectDirTraversal("\t"));
+  CPPUNIT_ASSERT(!util::detectDirTraversal("foo/bar"));
+  CPPUNIT_ASSERT(!util::detectDirTraversal("foo"));
+}
+
+void UtilTest::testEscapePath()
+{
+  CPPUNIT_ASSERT_EQUAL(std::string("foo%00bar%00%01"),
+                       util::escapePath(std::string("foo")+(char)0x00+
+                                        std::string("bar")+(char)0x00+
+                                        (char)0x01));
+#ifdef __MINGW32__
+  CPPUNIT_ASSERT_EQUAL(std::string("foo%5Cbar"), util::escapePath("foo\\bar"));
+#else // !__MINGW32__
+  CPPUNIT_ASSERT_EQUAL(std::string("foo\\bar"), util::escapePath("foo\\bar"));
+#endif // !__MINGW32__
+}
+
+void UtilTest::testGetCidrPrefix()
+{
+  struct in_addr in;
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "192.168.0.1", 16));
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.0"), std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "192.168.255.255", 17));
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.128.0"),std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "192.168.128.1", 16));
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.0"), std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "192.168.0.1", 32));
+  CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"), std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "192.168.0.1", 0));
+  CPPUNIT_ASSERT_EQUAL(std::string("0.0.0.0"), std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(util::getCidrPrefix(in, "10.10.1.44", 27));
+  CPPUNIT_ASSERT_EQUAL(std::string("10.10.1.32"), std::string(inet_ntoa(in)));
+
+  CPPUNIT_ASSERT(!util::getCidrPrefix(in, "::1", 32));
+}
+
+void UtilTest::testInSameCidrBlock()
+{
+  CPPUNIT_ASSERT(util::inSameCidrBlock("192.168.128.1", "192.168.0.1", 16));
+  CPPUNIT_ASSERT(!util::inSameCidrBlock("192.168.128.1", "192.168.0.1", 17));
+}
+
+void UtilTest::testIsUtf8String()
+{
+  CPPUNIT_ASSERT(util::isUtf8("ascii"));
+  // "Hello World" in Japanese UTF-8
+  CPPUNIT_ASSERT(util::isUtf8
+                 (util::fromHex("e38193e38293e381abe381a1e381afe4b896e7958c")));
+  // "World" in Shift_JIS
+  CPPUNIT_ASSERT(!util::isUtf8(util::fromHex("90a28a")+"E"));
+  // UTF8-2
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("c280")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("dfbf")));
+  // UTF8-3
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("e0a080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("e0bf80")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("e18080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("ec8080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("ed8080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("ed9f80")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("ee8080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("ef8080")));
+  // UTF8-4
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f0908080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f0bf8080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f1808080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f3808080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f4808080")));
+  CPPUNIT_ASSERT(util::isUtf8(util::fromHex("f48f8080")));
+
+  CPPUNIT_ASSERT(util::isUtf8(""));
+  CPPUNIT_ASSERT(!util::isUtf8(util::fromHex("00")));
+}
+
+void UtilTest::testNextParam()
+{
+  std::string s1 = "    :a  :  b=c :d=b::::g::";
+  std::pair<std::string::iterator, bool> r;
+  std::string name, value;
+  r = util::nextParam(name, value, s1.begin(), s1.end(), ':');
+  CPPUNIT_ASSERT(r.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("a"), name);
+  CPPUNIT_ASSERT_EQUAL(std::string(), value);
+
+  r = util::nextParam(name, value, r.first, s1.end(), ':');
+  CPPUNIT_ASSERT(r.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("b"), name);
+  CPPUNIT_ASSERT_EQUAL(std::string("c"), value);
+
+  r = util::nextParam(name, value, r.first, s1.end(), ':');
+  CPPUNIT_ASSERT(r.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("d"), name);
+  CPPUNIT_ASSERT_EQUAL(std::string("b"), value);
+
+  r = util::nextParam(name, value, r.first, s1.end(), ':');
+  CPPUNIT_ASSERT(r.second);
+  CPPUNIT_ASSERT_EQUAL(std::string("g"), name);
+  CPPUNIT_ASSERT_EQUAL(std::string(), value);
+
+  std::string s2 = "";
+  r = util::nextParam(name, value, s2.begin(), s2.end(), ':');
+  CPPUNIT_ASSERT(!r.second);
+
+  std::string s3 = "   ";
+  r = util::nextParam(name, value, s3.begin(), s3.end(), ':');
+  CPPUNIT_ASSERT(!r.second);
+
+  std::string s4 = ":::";
+  r = util::nextParam(name, value, s4.begin(), s4.end(), ':');
+  CPPUNIT_ASSERT(!r.second);
+}
+
 } // namespace aria2
